#!/usr/bin/python

"""
common.py - commonly used scene classes
"""

import os
import pygame
from pygame.locals import *
from gamedirector import *
from math import *

import resources

class Camera(object):
    def __init__(self,screen_size,stickyness=0.33):
        self.x = -1
        self.y = -1
        self.target = (-1,-1)
        self.w_view = screen_size[0]
        self.h_view = screen_size[1]
        self.stickyness = stickyness
    
    def SetTarget(self,target):
        self.target = target        
    
    def MoveCamera(self,focus):
        if (focus[0] - self.x) > self.stickyness*self.w_view/2:
            self.x = focus[0] - self.stickyness*self.w_view/2
        elif (focus[0] - self.x) < -self.stickyness*self.w_view/2:
            self.x = focus[0] + self.stickyness*self.w_view/2
        if (focus[1] - self.y) > self.stickyness*self.h_view/2:
            self.y = focus[1] - self.stickyness*self.h_view/2
        elif (focus[1] - self.y) < -self.stickyness*self.h_view/2:
            self.y = focus[1] + self.stickyness*self.h_view/2
        self.constrain()
    
    def PanCamera(self,vx,vy):
        speed = 12
        self.target = [-1,-1]
        self.x = self.x + speed*vx
        self.y = self.y + speed*vy
        self.constrain()
    
    def constrain(self):
        hit = False
        if self.x < self.w_view/2:
            self.x = self.w_view/2
            hit = True
        elif self.x > (self.xlim-self.w_view/2):
            self.x = self.xlim-self.w_view/2
            hit = True
        if self.y < self.h_view/2:
            self.y = self.h_view/2
            hit = True
        elif self.y > (self.ylim-self.h_view/2):
            self.y = self.ylim-self.h_view/2
            hit = True
        return hit
    
    def UpdateCamera(self,cursor_pos_screen):
        vx = 0
        vy = 0
        if cursor_pos_screen[0] == 0:
            vx = -1
        elif cursor_pos_screen[0] == self.w_view-1:
            vx = 1
        if cursor_pos_screen[1] == 0:
            vy = -1
        elif cursor_pos_screen[1] == self.h_view-1:
            vy = 1
        if abs(vx) > 0 or abs(vy) > 0:
            self.PanCamera(vx,vy)
        elif self.target[0] >= 0:
            range = sqrt(pow(self.target[0]-self.x,2)+pow(self.target[1]-self.y,2))
            if range < 2.0:
                self.x = self.target[0]
                self.y = self.target[1]
                self.target = [-1,-1]
            else:
                v = min(range/2.0,12.0)
                vx = v*(self.target[0]-self.x)/range
                vy = v*(self.target[1]-self.y)/range
                self.x = self.x + vx
                self.y = self.y + vy
        hit = self.constrain()
        #if hit:
        #    self.target = [-1,-1]

"""
class FadeInOut(object):
    def __init__(self,ticks):
        self.ticks = ticks
        self.ticks_elapsed = 0
        self.direction = 'none'
        self.alpha = 255
        self.finished_in = False
        self.finished_out = False
        self.musicfade = False
    
    def Update(self):
        if self.direction == 'in' and not self.finished_in:
            self.ticks_elapsed += 1
            if self.ticks_elapsed > self.ticks:
                self.finished_in = True
            else:
                self.alpha = 255*(1.0-(self.ticks_elapsed/float(self.ticks)))
                #self.alpha = 255*(1.0-((5*(self.ticks_elapsed/5))/float(self.ticks)))
        elif self.direction == 'out' and not self.finished_out:
            self.ticks_elapsed += 1
            if self.ticks_elapsed > self.ticks:
                self.finished_out = True
            else:
                self.alpha = 255*(self.ticks_elapsed/float(self.ticks))
                #self.alpha = 255*((5*(self.ticks_elapsed/5))/float(self.ticks))
                if self.musicfade:
                    pygame.mixer.music.set_volume(0.5*(1.0 - (self.ticks_elapsed/float(self.ticks))))
    
    def FadeIn(self):
        self.direction = 'in'
        self.ticks_elapsed = 0
        self.finished_in = False
        self.finished_out = False
    
    def FadeOut(self,musicfade):
        self.direction = 'out'
        self.ticks_elapsed = 0
        self.finished_out = False
        self.finished_in = False
        self.musicfade = musicfade
    
    def Reset(self):
        self.ticks_elapsed = 0
        self.direction = 'none'
        self.alpha = 255
        self.finished_in = False
        self.finished_out = False
        self.musicfade = False
"""

class FadeInOut(object):
    def __init__(self,ticks,smooth=False):
        self.ticks = ticks
        self.ticks_elapsed = 0
        self.direction = 'none'
        self.alpha = 255
        self.finished_in = False
        self.finished_out = False
        self.musicfade = False
        self.smooth = smooth
        self.running = False
    
    def Update(self):
        if not self.running:
            return
        if self.direction == 'in' and not self.finished_in:
            self.ticks_elapsed += 1
            if self.ticks_elapsed > self.ticks:
                self.finished_in = True
            else:
                if self.smooth:
                    self.alpha = 255*(1.0-(self.ticks_elapsed/float(self.ticks)))
                else:
                    self.alpha = 255*(1.0-((5*(self.ticks_elapsed/5))/float(self.ticks)))
        elif self.direction == 'out' and not self.finished_out:
            self.ticks_elapsed += 1
            if self.ticks_elapsed > self.ticks:
                self.finished_out = True
            else:
                if self.smooth:
                    self.alpha = 255*(self.ticks_elapsed/float(self.ticks))
                else:
                    self.alpha = 255*((5*(self.ticks_elapsed/5))/float(self.ticks))
                if self.musicfade:
                    pygame.mixer.music.set_volume(0.5*(1.0 - (self.ticks_elapsed/float(self.ticks))))
    
    def FadeIn(self):
        self.direction = 'in'
        self.ticks_elapsed = 0
        self.finished_in = False
        self.finished_out = False
        self.running = True
    
    def FadeOut(self,musicfade=False):
        self.direction = 'out'
        self.ticks_elapsed = 0
        self.finished_out = False
        self.finished_in = False
        self.musicfade = musicfade
        self.running = True
    
    def Reset(self):
        self.ticks_elapsed = 0
        self.direction = 'none'
        self.alpha = 255
        self.finished_in = False
        self.finished_out = False
        self.musicfade = False
        self.running = False

class MsgDialogue(object):
    def __init__(self):
        self.msgkey = None
        self.active_to = 0
        self.mentor = False
    
    def DisplayMsg(self, k, mentor=False, mentor_ind=0):
        self.msgkey = k
        self.active = True
        self.active_to = 100
        self.mentor = mentor
        self.mentor_ind = mentor_ind
        self.msgind = 0
    
    def update(self):
        if self.mentor == False:
            self.active_to = max(self.active_to-1,0)
    
    def render(self, screen):
        if self.active_to > 0:
            if self.mentor == True:
                screen.blit(resources.status_message['background2'], (24,3))
                screen.blit(resources.mentor_data['panel_facementor'], (3, 3))
                if type(self.mentor_ind) == type(""):
                    screen.blit(resources.mentor_messages['startmsgsurf'][self.mentor_ind][self.msgind], (48, 3))
                else:
                    screen.blit(resources.mentor_messages['msgsurf'][self.mentor_ind][self.msgind], (48, 3))
            else:
                screen.blit(resources.status_message['background'], (24,12))
                screen.blit(resources.status_message[self.msgkey], (0, 0))

class MenuButton(object):
    def __init__(self,pos,rect_on,rect_off,surf_on,surf_off):
        self.pos = pos
        self.hover = False
        self.rect_on = rect_on
        self.rect_off = rect_off
        self.surf_on = surf_on
        self.surf_off = surf_off
    
    def Draw(self, screen):
        if self.hover:
            screen.blit(self.surf_on, self.pos)
        else:
            screen.blit(self.surf_off, self.pos)

class MenuList(object):
    def __init__(self,buttons):
        self.buttons = buttons
        self.currentbutton = None
        self.cursor_pos_screen = pygame.mouse.get_pos()
    
    def SetMousePos(self):
        self.cursor_pos_screen = pygame.mouse.get_pos()
    
    def Reset(self):
        self.currentbutton = None
        self.cursor_pos_screen = pygame.mouse.get_pos()
    
    def SetGoFunc(self,button_ind,func):
        self.buttons[button_ind].go_func = func
    
    def ProcessKeyEvent(self,event):
        if event.type == MOUSEBUTTONDOWN:
            self.SetMousePos()
            if not self.currentbutton == None:
                self.currentbutton.go_func(self.switchtoargs)
        elif event.type == MOUSEMOTION:
            self.SetMousePos()
            self.currentbutton = None
            for button in self.buttons:
                if self.cursor_pos_screen[0] > (button.rect_off.x+button.pos[0]) and self.cursor_pos_screen[0] < (button.rect_off.x+button.rect_off.w+button.pos[0]) and \
                    self.cursor_pos_screen[1] > (button.rect_off.y+button.pos[1]) and self.cursor_pos_screen[1] < (button.rect_off.y+button.rect_off.h+button.pos[1]):
                    button.hover = True
                    self.currentbutton = button
                else:
                    button.hover = False
    
    def Draw(self, screen):
        for b in self.buttons:
            b.Draw(screen)

class ClickOptionsScreen(GameScene):
    def __init__(self, director, window_size, fadelevel):
        super(ClickOptionsScreen, self).__init__(director)
        self.window_size = window_size
        self.cursorsize = 24
        
        # Background
        self.background = pygame.Surface(window_size)
        self.background.fill((0,0,0))
        self.background.convert()
        self.fadelevel = fadelevel
    
    def on_switchto(self, switchtoargs):
        self.switchtoargs = switchtoargs
        self.menulist.switchtoargs = switchtoargs
        self.menulist.Reset()
        
        # Setup background
        if self.fadelevel > 0:
            backfade = pygame.Surface(self.background.get_size())
            backfade.fill((0,0,0))
            backfade.convert()
            backfade.set_alpha(self.fadelevel)
            self.background.blit(backfade, (0,0))
        
    def SetMousePos(self):
        self.cursor_pos_screen = pygame.mouse.get_pos()
        
    def on_update(self):
        pass
    
    def on_event(self, events):
        for event in events:
            if event.type == KEYDOWN and event.key == K_ESCAPE:
                self.EscapeFunc(self.switchtoargs)
            else:
                self.menulist.ProcessKeyEvent(event)
    
    def on_draw(self, screen):
        screen.blit(self.background, (0,0))
        self.menulist.Draw(screen)
        self.SetMousePos()
        tilecoords = resources.mouse_spritesheet_coords['hover'][resources.mouse_spriteseq['hover'][0]]
        screen.blit(resources.mouse_spritesheet, (self.cursor_pos_screen[0]-self.cursorsize/2,self.cursor_pos_screen[1]-self.cursorsize/2), area=tilecoords)

class CutScene(GameScene):
    def __init__(self, director, window_size):
        super(CutScene, self).__init__(director)
        self.window_size = window_size
        
        # Background
        self.background = pygame.Surface(window_size)
        self.background.fill((0,0,0))
        self.background.convert()
        
        self.midground = pygame.Surface(window_size)
        self.midground.fill((0,0,0))
        self.midground.convert()
        
        self.fadebackground = pygame.Surface(window_size)
        self.fadebackground.fill((0,0,0))
        self.fadebackground.convert()
        
        self.fade = FadeInOut(30)
        self.fade_text = FadeInOut(15)
        
        self.go_next = False
        
    def on_switchto(self, switchtoargs):
        
        self.sequence = switchtoargs[0] # [ [bgid, [textid1, textid2, ...] ]
        self.musicid = switchtoargs[1]
        self.next_scene_type = switchtoargs[2]
        self.next_scene_switchtoargs = switchtoargs[3]
        
        self.step = 0
        self.textstep = -1
        
        # Check music
        if self.musicid == 'none':
            pygame.mixer.music.stop()
        if not self.musicid == 'none' and not self.musicid == resources.cutscenecurrentmusic:
            pygame.mixer.music.stop()
            resources.cutscenecurrentmusic = self.musicid
            pygame.mixer.music.load(resources.musicpaths[self.musicid])
            pygame.mixer.music.set_volume(0.5)
            pygame.mixer.music.play(-1)
        
        self.fade.FadeIn()
        
    def on_update(self):
        
        # Check for return to game
        self.fade.Update()
        self.fade_text.Update()
        if self.fade.direction == 'in' and self.fade.finished_in and not self.fade_text.running:
            if len(self.sequence[self.step][1]) > 0:
                self.textstep = 0
                self.fade_text.FadeIn()
        if self.go_next:
            if len(self.sequence[self.step][1]) > 0: # any text?
                if self.fade.direction == 'in' and self.fade_text.direction == 'in' and self.fade_text.running:
                    self.fade_text.FadeOut()
            else:
                if self.fade.direction == 'in':                    
                    if len(self.next_scene_switchtoargs) == 4:
                        if not self.musicid == self.next_scene_switchtoargs[1]:
                            self.fade.FadeOut(musicfade=True)
                        else:
                            self.fade.FadeOut()
                    else:
                        self.fade.FadeOut(musicfade=True)
            self.go_next = False
        if self.fade_text.finished_out: # next text
            self.fade_text.Reset()
            self.textstep += 1
            if self.textstep == len(self.sequence[self.step][1]):
                self.textstep = -1
                if self.step == len(self.sequence)-1:
                    self.fade.FadeOut(True)
                else:
                    self.fade.FadeOut()
            else:
                self.fade_text.FadeIn()
        if self.fade.finished_out: # next cut
            self.step += 1
            if self.step == len(self.sequence):
                self.director.change_scene(self.next_scene_type,self.next_scene_switchtoargs)
            else:
                self.fade.Reset()
                self.fade.FadeIn()
        
    def on_event(self, events):
        for event in events:
            if event.type == KEYDOWN and event.key == K_ESCAPE:
                pygame.mixer.music.stop()
                self.director.change_scene(self.next_scene_type,self.next_scene_switchtoargs)
            if event.type == KEYDOWN or event.type == MOUSEBUTTONDOWN:
                self.go_next = True
    
    def render_animation(self, screen):
        raise NotImplementedError("render_animation abstract method must be defined in subclass.")
    
    def render_bg(self, screen):
        bgdata = self.sequence[self.step][0]
        if isinstance(bgdata,list):
            render_animation(screen)
            return
        surf = resources.cutscenesurfs[bgdata]
        pos = ((self.window_size[0]/2)-(surf.get_width()/2), (self.window_size[1]/2)-(surf.get_height()/2))
        screen.blit(surf, pos)
    
    def render_text(self, screen):
        textdata = self.sequence[self.step][1][self.textstep]
        surf = resources.cutscenesurfs[textdata]
        pos = ((self.window_size[0]/2)-(surf.get_width()/2), (self.window_size[1]/2)-(surf.get_height()/2))
        surf.set_alpha(255-self.fade_text.alpha)
        screen.blit(surf, pos)
    
    def on_draw(self, screen):
        screen.blit(self.background, (0, 0))
        self.render_bg(screen)
        if self.textstep >= 0:
            self.render_text(screen)
        self.fadebackground.set_alpha(self.fade.alpha)
        screen.blit(self.fadebackground, (0, 0))

class CutSceneShipExplode(CutScene):
    def __init__(self, director, window_size):
        super(CutSceneShipExplode, self).__init__(director,window_size)
        self.timer = 0
    
    def on_switchto(self, switchtoargs):
        
        self.sequence = switchtoargs[0] # [ [bgid, [textid1, textid2, ...] ]
        self.musicid = switchtoargs[1]
        self.next_scene_type = switchtoargs[2]
        self.next_scene_switchtoargs = switchtoargs[3]
        
        self.step = 0
        self.textstep = -1
        
        # Check music
        if self.musicid == 'none':
            pygame.mixer.music.stop()
        if not self.musicid == 'none' and not self.musicid == resources.cutscenecurrentmusic:
            pygame.mixer.music.stop()
            resources.cutscenecurrentmusic = self.musicid
            pygame.mixer.music.load(resources.musicpaths[self.musicid])
            pygame.mixer.music.set_volume(0.5)
            pygame.mixer.music.play()
        
        self.fade.FadeIn()
    
    def on_update(self):
        super(CutSceneShipExplode, self).on_update()
        self.timer += 1
        if self.timer > 210:
            self.go_next = True
    
    def on_draw(self, screen):
        screen.blit(self.background, (0, 0))
        self.render_bg(screen)
        if self.textstep >= 0:
            self.render_text(screen)
        radius_a = self.timer/4
        radius_b = self.timer/8
        if self.timer < 140:
            radius2 = 0
        else:
            radius2 = (self.timer-140)/2
        pygame.draw.ellipse(screen, (208,208,208), ((320-radius_a,240-radius_b),(2*radius_a,2*radius_b)))
        if radius2 > 0:
            pygame.draw.ellipse(screen, (0,0,0), ((320-radius2,240-(radius2/2)),(2*radius2,radius2)))
        self.fadebackground.set_alpha(self.fade.alpha)
        screen.blit(self.fadebackground, (0, 0))
    
        