#!/usr/bin/python

"""
game.py - main in-game scene classes
"""

import os
import pygame
from pygame.locals import *
from gamedirector import *
import random

import resources
import tilemap
import units
import path

from common import Camera, FadeInOut, MsgDialogue, MenuButton, MenuList, ClickOptionsScreen
from resources import MovingStarField

class HUD(object):
    def __init__(self,parent):
        self.parent = parent
        self.ticks = 0
        self.nticks = 10
        self.frame = 0
        self.crewalive = 0
        self.crewdead = 0
        self.crewescapepod = 0
    
    def Update(self):
        self.crewalive = 0
        self.crewdead = 0
        self.crewescapepod = 0
        for unit in self.parent.unitlist:
            if unit.alive == True:
                self.crewalive += 1
                if unit.escapepod == True:
                    self.crewescapepod += 1
            else:
                self.crewdead += 1
        if self.parent.gamestate == 'playing' and self.crewalive == 0:
            self.parent.gamestate = 'failure'
            self.parent.quitcountdown = 60
        self.ticks += 1
        if (self.ticks % self.nticks) == 0:
            self.frame = (self.frame+1)%4
    
    def Draw(self,screen):
        screen.blit(resources.hud_data['panel_bg'], (24,417))
        screen.blit(resources.hud_data['panel_abandon'], resources.hud_data['panel_abandon_coords'])
        if self.crewescapepod > 0:
            screen.blit(resources.hud_data['panel_launch'], resources.hud_data['panel_launch_coords'])
            pygame.draw.rect(screen,resources.hud_data['launchcolours'][self.frame],((171*3+22,10*3+415),(29*3-2,10*3-2)),3)
        for i in xrange(len(self.parent.unitlist)):
            id = self.parent.unitlist[i].charid
            if self.parent.unitlist[i].type == 'unit':
                surfref = resources.hud_data['panel_facesunits'][id]
            else:
                surfref = resources.hud_data['panel_facesastro'][id]
            screen.blit(surfref, (1*3+24+i*11*3+3,9*3+417+3))
            if self.parent.unitlist[i].escapepod == True:
                screen.blit(resources.hud_data['panel_statusbox']['safe'], (1*3+24+i*11*3,9*3+417))
            elif self.parent.unitlist[i].alive == True:
                screen.blit(resources.hud_data['panel_statusbox']['alive'], (1*3+24+i*11*3,9*3+417))
            else:
                screen.blit(resources.hud_data['panel_statusbox']['dead'], (1*3+24+i*11*3,9*3+417))
            if (self.frame % 2) == 0 and self.parent.control.unit_id == i:
                pygame.draw.rect(screen,(205,205,205),((1*3+24+i*11*3+1,9*3+417+1),(10*3-2,10*3-2)),3)
        """
        surf1 = resources.gamefont_green.RenderSentence("Crew in escape pod: %d of %d"%(self.crewescapepod,self.crewalive), 160, 48, align='vcentre')
        surf2 = resources.gamefont_red.RenderSentence("(%d dead)"%(self.crewdead), 160, 48, align='vcentre')
        screen.blit(surf1,(60,420))
        if self.crewdead > 0:
            screen.blit(surf2,(240,420))
        if self.crewalive == 0:
            screen.blit(resources.hud_data['failure'],(400,420))
        elif self.crewalive == self.crewescapepod:
            screen.blit(resources.hud_data['launch'],(400,420))
            pygame.draw.rect(screen,resources.hud_data['launchcolours'][self.frame],resources.hud_data['launchpos'],3)
        """

class AssistMsg(object):
    def __init__(self,parent):
        self.parent = parent
        self.active = False
        self.msgkey = None
        self.type = None
        self.tile = None
        self.assoc_tiles = []
    
    def Update(self,tile):
        unit_ids = self.parent.h_tiledlayers.FindObjects(tile,'units')
        if len(unit_ids) > 0: # hovering a unit
            if self.parent.h_unitlist[unit_ids[0]].alive == True:
                self.active = True
                self.tile = tile
                if self.parent.h_unitlist[unit_ids[0]].escapepod == True:
                    self.msgkey = 'crewsafe'
                elif self.parent.h_unitlist[unit_ids[0]].type == 'unit':
                    if self.parent.h_unitlist[unit_ids[0]].numpowerpacks > 0:
                        self.msgkey = 'crewunitpowerpack'
                    else:
                        self.msgkey = 'crewunit'
                elif self.parent.h_unitlist[unit_ids[0]].type == 'astro':
                    if self.parent.h_unitlist[unit_ids[0]].numpowerpacks > 0:
                        self.msgkey = 'crewastropowerpack'
                    else:
                        self.msgkey = 'crewastro'
                self.assoc_tiles = []
                return
        ids = self.parent.h_tiledlayers.FindObjects(tile,'goal')
        if len(ids) > 0: # hovering a goal
            self.active = True
            self.tile = tile
            self.msgkey = 'goal'
            self.assoc_tiles = []
            return
        ids = self.parent.h_tiledlayers.FindObjects(tile,'fire')
        if len(ids) > 0: # hovering a fire
            self.active = True
            self.tile = tile
            self.msgkey = 'fire'
            self.assoc_tiles = []
            return
        ids = self.parent.h_tiledlayers.FindObjects(tile,'key')
        if len(ids) > 0: # hovering a powerchip
            self.active = True
            self.tile = tile
            self.msgkey = 'powerchip'
            self.assoc_tiles = []
            return
        ids = self.parent.h_tiledlayers.FindObjects(tile,'door')
        if len(ids) > 0: # hovering a door
            self.active = True
            self.tile = tile
            self.msgkey = 'door'
            self.assoc_tiles = []
            return
        ids = self.parent.h_tiledlayers.FindObjects(tile,'pump')
        if len(ids) > 0: # hovering a pump
            self.active = True
            self.tile = tile
            self.msgkey = 'airpump'
            self.assoc_tiles = []
            return
        ids = self.parent.h_tiledlayers.FindObjects(tile,'switch')
        if len(ids) > 0: # hovering a switch
            self.active = True
            self.tile = tile
            powerstate = self.parent.h_tiledlayers.switchcontrol.switches[ids[0]].powerstate
            type = self.parent.h_tiledlayers.switchcontrol.switches[ids[0]].type
            state = self.parent.h_tiledlayers.switchcontrol.switches[ids[0]].state
            if self.parent.unit_id < 0:
                if type == 'powerplug':
                    if state == -1:
                        self.msgkey = 'powerplug'
                    else:
                        self.msgkey = 'powerplugempty'
                else:
                    if powerstate == 1:
                        self.msgkey = 'switch'
                    else:
                        self.msgkey = 'switchnopower'
                self.assoc_tiles = []
            else:
                h_assoc = self.parent.h_tiledlayers.switchcontrol.switches[ids[0]].h_assoc
                if self.parent.h_tiledlayers.switchcontrol.switches[ids[0]].type == 'door':
                    doorstate = h_assoc[0].state
                    if doorstate == 1:
                        self.msgkey = 'closedoor'
                    else:
                        self.msgkey = 'opendoor'
                    self.assoc_tiles = []
                    for h in h_assoc:
                        self.assoc_tiles.append(h.tilelist[0])
                elif self.parent.h_tiledlayers.switchcontrol.switches[ids[0]].type == 'pump':
                    pumpstate = h_assoc[0].state
                    if pumpstate == True:
                        self.msgkey = 'depressurise'
                    else:
                        self.msgkey = 'repressurise'
                    self.assoc_tiles = []
                    for h in h_assoc:
                        self.assoc_tiles.append(h.tile1)
                elif self.parent.h_tiledlayers.switchcontrol.switches[ids[0]].type == 'powerplug':
                    if state == -1:
                        self.msgkey = 'powerplug'
                    else:
                        if self.parent.h_unitlist[self.parent.unit_id].numpowerpacks > 0:
                            self.msgkey = 'powerchipinsert'
                        else:
                            self.msgkey = 'powerplugempty'
                    self.assoc_tiles = []
                    for h in h_assoc:
                        self.assoc_tiles.append(h.tile1)
            return
        self.active = False
        
    def DrawAssistMsg(self, screen):
        if self.active == True:
            ts = self.parent.h_tiledlayers.tilesize
            msx = self.parent.h_tiledlayers.map_size[0]
            for h in self.assoc_tiles:
                x = (h%msx)*ts
                y = (h/msx)*ts
                assocpos = (x-self.parent.h_camera.x+self.parent.h_camera.w_view/2,y-self.parent.h_camera.y+self.parent.h_camera.h_view/2)
                pygame.draw.rect(screen,(0,255,0),(assocpos,(ts,ts)),3)
            x = (self.tile%msx)*ts + (ts/2) - 80
            y = (self.tile/msx)*ts + (ts/2) - 24
            screen.blit(resources.assist_message[self.msgkey], \
                (x-self.parent.h_camera.x+self.parent.h_camera.w_view/2,y-self.parent.h_camera.y+self.parent.h_camera.h_view/2))

class KeyControl():
    
    def __init__(self,window_size,parent):
        self.window_size = window_size
        self.parent = parent
        self.cursor_pos_screen = pygame.mouse.get_pos()
        self.cursor_pos = (0,0)
        self.cursorsize = 24
        self.unit_id = -1
        self.assistmsg = AssistMsg(self)
        
    def set_handles(self,handles):
        (self.h_camera, self.h_unitlist, self.h_tiledlayers) = handles
    
    def SetMousePos(self):
        self.cursor_pos_screen = pygame.mouse.get_pos()
        self.cursor_pos = (int(self.cursor_pos_screen[0]+self.h_camera.x-self.h_camera.w_view/2), \
            int(self.cursor_pos_screen[1]+self.h_camera.y-self.h_camera.h_view/2))
    
    def ProcessKeyEvent(self,event):
        if event.type == KEYDOWN and event.key == K_z: # debug create new unit
            self.SetMousePos()
            self.h_unitlist.append(units.MouseControlableUnit(len(self.h_unitlist),'unit'))
            self.h_unitlist[-1].set_handles((self.h_camera, self.h_tiledlayers))
            tsize = self.h_tiledlayers.tilesize
            gtile = self.h_tiledlayers.map_size[0]*(self.cursor_pos[1]/tsize)+(self.cursor_pos[0]/tsize)
            self.h_unitlist[-1].SetTile(gtile)
            self.h_tiledlayers.InsertObj(gtile,self.h_unitlist[-1].id)
        elif event.type == KEYUP:
            pass
        elif event.type == MOUSEMOTION:
            self.SetMousePos()
            tsize = self.h_tiledlayers.tilesize
            tile = self.h_tiledlayers.map_size[0]*(self.cursor_pos[1]/tsize)+(self.cursor_pos[0]/tsize)
            self.assistmsg.Update(tile)
        elif event.type == MOUSEBUTTONDOWN:
            if self.parent.msgdialogue.mentor == True:
                self.parent.msgdialogue.msgind += 1
                mentor_ind = self.parent.msgdialogue.mentor_ind
                msgind = self.parent.msgdialogue.msgind
                if type(mentor_ind) == type(""):
                    mlist = resources.mentor_messages['startmsg'][mentor_ind]
                else:
                    mlist = resources.mentor_messages['msg'][mentor_ind]
                if msgind == len(mlist):
                    self.parent.msgdialogue.mentor = False
                    self.parent.msgdialogue.active = False
                    self.parent.msgdialogue.active_to = 0
                return
            if self.parent.gamestate == 'playing':
                self.SetMousePos()
                tsize = self.h_tiledlayers.tilesize
                gtile = self.h_tiledlayers.map_size[0]*(self.cursor_pos[1]/tsize)+(self.cursor_pos[0]/tsize)
                # first check if launching escape pod
                if self.parent.hud.crewescapepod > 0:
                    rect = ((171*3+22,10*3+415),(29*3-2,10*3-2))
                    if self.cursor_pos_screen[0] > rect[0][0] and self.cursor_pos_screen[0] < (rect[0][0]+rect[1][0]) and \
                        self.cursor_pos_screen[1] > rect[0][1] and self.cursor_pos_screen[1] < (rect[0][1]+rect[1][1]):
                        self.parent.gamestate = 'complete'
                        self.parent.tiledlayers.escapepod.launch = True
                        self.parent.tiledlayers.escapepod.surf.blit(resources.tiles, ((8*3,32*3)), area=resources.tiles_coords[208]) # close hatch!
                        self.parent.quitcountdown = 60
                        self.unit_id = -1
                # check for abandon button
                if self.parent.gamestate == 'playing':
                    rect = ((133*3+21,10*3+414),(36*3,10*3))
                    if self.cursor_pos_screen[0] > rect[0][0] and self.cursor_pos_screen[0] < (rect[0][0]+rect[1][0]) and \
                        self.cursor_pos_screen[1] > rect[0][1] and self.cursor_pos_screen[1] < (rect[0][1]+rect[1][1]):
                        self.parent.draw(self.parent.h_pausescene.background,ignore_cursor=True)
                        self.parent.director.change_scene('pausegame', self.parent.tiledlayers.level_id)
                    else: # continue checking
                        # first check if selecting new unit
                        unit_ids = self.h_tiledlayers.FindObjects(gtile,'units')
                        keepgoing = False
                        if len(unit_ids) > 0: # selecting a unit
                            if self.h_unitlist[unit_ids[0]].alive == True:
                                self.unit_id = unit_ids[0]
                                keepgoing = True
                        if len(unit_ids) > 0 and keepgoing == True:
                            pass
                            #self.h_camera.SetTarget([self.h_unitlist[self.unit_id].x,self.h_unitlist[self.unit_id].y])
                        else: # not selecting unit, check if activating switch
                            switch_ids = self.h_tiledlayers.FindObjects(gtile,'switch') # check if hitting a switch
                            if len(switch_ids) > 0: # hitting a switch
                                if self.unit_id >= 0: # have a unit selected
                                    if self.h_tiledlayers.switchcontrol.switches[switch_ids[0]].powerstate == 1: # switch has power
                                        switchtype = self.h_tiledlayers.switchcontrol.switches[switch_ids[0]].type
                                        unitnpowerchips = self.h_unitlist[self.unit_id].numpowerpacks
                                        if switchtype == 'powerplug' and unitnpowerchips == 0:
                                            self.parent.msgdialogue.DisplayMsg("powerplugnochip")
                                        else:
                                            unit = self.h_unitlist[self.unit_id]
                                            unit.task = [] # reset task for now
                                            ptile = self.h_tiledlayers.map_size[0]*(unit.y/tsize)+(unit.x/tsize)
                                            switch_stand_tile = self.h_tiledlayers.switchcontrol.switches[switch_ids[0]].tile2
                                            unit.path = self.h_planner.astar_path(ptile, switch_stand_tile)
                                            if unit.path == []:
                                                self.parent.msgdialogue.DisplayMsg("unitnomove")
                                            else:
                                                unit.task = [switch_stand_tile,switch_ids[0]]
                                    else:
                                        self.parent.msgdialogue.DisplayMsg("switchpower")
                                else:
                                    self.parent.msgdialogue.DisplayMsg("unitnoselect")
                            elif self.unit_id >= 0: # directing a unit somewhere, not a switch
                                unit = self.h_unitlist[self.unit_id]
                                unit.task = [] # reset task for now
                                ptile = self.h_tiledlayers.map_size[0]*(unit.y/tsize)+(unit.x/tsize)
                                unit.path = self.h_planner.astar_path(ptile, gtile)
                                if unit.path == []:
                                    self.parent.msgdialogue.DisplayMsg("unitnomove")
        elif event.type == MOUSEBUTTONUP:
            pass
            
    def Draw(self,screen):
        if self.unit_id >= 0: # selected unit
            #unitpos = (self.h_unitlist[self.unit_id].x-18-self.h_camera.x+self.h_camera.w_view/2, self.h_unitlist[self.unit_id].y-18-6-self.h_camera.y+self.h_camera.h_view/2)
            x = self.h_unitlist[self.unit_id].x
            y = self.h_unitlist[self.unit_id].y
            ts = self.h_unitlist[self.unit_id].tilesize
            unitpos = (x-(ts/2)-self.h_camera.x+self.h_camera.w_view/2-6,y-(ts/2)-self.h_camera.y+self.h_camera.h_view/2-6)
            pygame.draw.rect(screen,(255,0,0),(unitpos,(ts+12,ts+12)),3)
            
    def DrawCursor(self,screen):
        tilecoords = resources.mouse_spritesheet_coords['hover'][resources.mouse_spriteseq['hover'][0]]
        screen.blit(resources.mouse_spritesheet, (self.cursor_pos_screen[0]-self.cursorsize/2,self.cursor_pos_screen[1]-self.cursorsize/2), area=tilecoords)

class TrackingData(object):
    def __init__(self):
        self.current_level = resources.levellist[0]
        self.totalcrewrescued = 0
        self.totalcrewabandoned = 0
        self.totalcrewdead = 0
    
    def LoadProgressData(self):
        f = open(self.savepath, "r");
        self.current_level = f.readline().split()[0]
        self.totalcrewrescued = bool(int(f.readline().split()[0]))
        self.totalcrewabandoned = bool(int(f.readline().split()[0]))
        self.totalcrewdead = bool(int(f.readline().split()[0]))
        f.close()
    
    def SaveProgressData(self):
        f = open(self.savepath, "w");
        f.write('%s\n'%(self.current_level))
        f.write('%d\n'%(int(self.totalcrewrescued)))
        f.write('%d\n'%(int(self.totalcrewabandoned)))
        f.write('%d\n'%(int(self.totalcrewdead)))
        f.close()
    
    def Reset(self):
        self.current_level = resources.levellist[0]
        self.totalcrewrescued = 0
        self.totalcrewabandoned = 0
        self.totalcrewdead = 0

class MainGame(GameScene):
    def __init__(self, director, window_size):
        super(MainGame, self).__init__(director)
        self.window_size = window_size
        
        # frame rate recording
        self.avgframerate = -1
        self.frsamples = 0
        
        # fade in/out
        self.fade = FadeInOut(30)
        
        # Music
        self.current_music = 'none'
        
        # Background
        self.background = pygame.Surface(window_size)
        self.background.fill((0,0,0))
        self.background.convert()
        
        # Level tracking data
        self.trackingdata = TrackingData()
    
    def on_switchto(self, switchtoargs):
        
        lvlreset = switchtoargs[0]
        if not lvlreset:
            return
        level_id = switchtoargs[1]
        
        self.trackingdata.current_level = level_id
        self.trackingdata.SaveProgressData()
        
        self.gamestate = 'playing'
        
        # re-draw stars randomly
        self.starrybackdrop = pygame.Surface(self.window_size)
        self.starrybackdrop.fill((0,0,0))
        for i in xrange(35):
            x = random.randint(-5,634)
            y = random.randint(-5,474)
            self.starrybackdrop.blit(resources.singlestar, (x,y), area=(9,0,9,9))
        for i in xrange(10):
            x = random.randint(-5,634)
            y = random.randint(-5,474)
            self.starrybackdrop.blit(resources.singlestar, (x,y), area=(0,0,9,9))
        
        # Initialise objects
        self.msgdialogue = MsgDialogue()
        self.hud = HUD(self)
        self.camera = Camera(self.window_size)
        self.control = KeyControl(self.window_size,self)
        self.tiledlayers = tilemap.TiledLayers(self)
        self.unitlist = []
        #self.unitlist.append(units.MouseControlableUnit(0))
        
        # load level data
        self.tiledlayers.set_handles((self.camera, self.unitlist, self.control, self.msgdialogue))
        self.tiledlayers.load_level(level_id)
        
        # init other stuff
        self.planner = path.Planner(self.tiledlayers.map_size,self.tiledlayers.tilesize,self.tiledlayers.occlayer)
        self.control.h_planner = self.planner
        
        # set cross handles
        self.control.set_handles((self.camera, self.unitlist, self.tiledlayers))
        for unit in self.unitlist:
            unit.h_planner = self.planner
        self.camera.MoveCamera([self.unitlist[0].x,self.unitlist[0].y])
        
        # Check music
        self.current_music = resources.initleveldata[level_id].music
        pygame.mixer.music.stop()
        if not self.current_music == 'none':
            pygame.mixer.music.load(resources.musicpaths[self.current_music])
            pygame.mixer.music.set_volume(0.5)
            pygame.mixer.music.play(-1)
        
        # Fade in game
        self.background.fill((0,0,0))
        self.fade.FadeIn()
        
    def on_update(self):
        
        # framerate tracking
        self.frsamples += 1
        if self.frsamples == 1:
            self.avgframerate = self.director.framerate
        else:
            self.avgframerate = self.avgframerate + (self.director.framerate - self.avgframerate)/(self.frsamples)
        
        # Update objects
        self.msgdialogue.update()
        self.hud.Update()
        self.tiledlayers.Update()
        for unit in self.unitlist:
            unit.UpdateMotion()
        self.camera.UpdateCamera(self.control.cursor_pos_screen)
        
        # Control fade in/out, look for end game cues
        self.fade.Update()
        if self.gamestate == 'complete' or self.gamestate == 'failure':
            self.quitcountdown -= 1
            if self.quitcountdown == 0 and self.fade.direction == 'in':
                musicfade = True
                self.fade.FadeOut(musicfade)
            if self.fade.finished_out:
                self.fade.Reset()
                if self.gamestate == 'complete':
                    # Update tracking data
                    self.trackingdata.totalcrewrescued += self.hud.crewescapepod
                    self.trackingdata.totalcrewabandoned += self.hud.crewescapepod-self.hud.crewalive
                    self.trackingdata.totalcrewdead += self.hud.crewdead
                    trackdat = [self.hud.crewescapepod,self.hud.crewalive,self.hud.crewdead,self.trackingdata.totalcrewrescued]
                    ind = resources.levellist.index(self.tiledlayers.level_id)
                    if ind == (len(resources.levellist)-1):
                        self.director.change_scene('summaryscreen', [trackdat, -1])
                    else:
                        if self.tiledlayers.level_id == "level_tutorial":
                            self.director.change_scene('summaryscreen', [trackdat, 'chapter2scene1'])
                        elif self.tiledlayers.level_id == "level_medium001":
                            self.director.change_scene('summaryscreen', [trackdat, 'chapter3scene1'])
                        elif self.tiledlayers.level_id == "level_solo":
                            self.director.change_scene('summaryscreen', [trackdat, 'chapter4scene1'])
                        elif self.tiledlayers.level_id == "level_twosections":
                            self.director.change_scene('summaryscreen', [trackdat, 'chapter5scene1'])
                        else:
                            self.director.change_scene('summaryscreen', [trackdat, resources.levellist[ind+1]])
                else:
                    self.director.change_scene('maingame', [True, self.tiledlayers.level_id])
    
    def on_event(self, events):
        for event in events:
            if event.type == KEYDOWN and event.key == K_ESCAPE: # pausing game
                self.draw(self.h_pausescene.background,ignore_cursor=True)
                self.director.change_scene('pausegame', self.tiledlayers.level_id)
            elif event.type in (KEYDOWN,KEYUP,MOUSEBUTTONDOWN,MOUSEBUTTONUP,MOUSEMOTION):
                self.control.ProcessKeyEvent(event)
    
    def draw(self, screen, ignore_cursor=False):
        screen.blit(self.starrybackdrop, (0, 0))
        #screen.blit(resources.backgroundstars, (0, 0))
        self.tiledlayers.RenderTileLayer(screen)
        self.tiledlayers.escapepod.Draw(screen)
        self.tiledlayers.doorcontrol.DrawDoors(screen)
        self.tiledlayers.vacpumpcontrol.DrawVacPumps(screen)
        self.tiledlayers.switchcontrol.DrawSwitches(screen)
        self.tiledlayers.powerpackcontrol.DrawPowerPacks(screen)
        for unit in self.unitlist:
            if self.tiledlayers.escapepod.launch == True:
                unit.Draw(screen,drawtoescapepod=True)
            else:
                unit.Draw(screen)
        self.tiledlayers.firecontrol.DrawFires(screen)
        self.tiledlayers.doorcontrol.DrawDoorFrames(screen)
        #self.tiledlayers.RenderAirLayer(screen)
        self.msgdialogue.render(screen)
        self.control.Draw(screen)
        if self.gamestate == 'playing':
            self.control.assistmsg.DrawAssistMsg(screen)
        self.hud.Draw(screen)
        if ignore_cursor == False:
            self.control.DrawCursor(screen)
    
    def on_draw(self, screen):
        self.draw(screen)
        self.background.set_alpha(self.fade.alpha)
        screen.blit(self.background, (0, 0))

class PauseGame(ClickOptionsScreen):    
    
    def __init__(self, director, window_size, fadelevel):
        super(PauseGame, self).__init__(director,window_size,fadelevel)
        pos = resources.pausemenu_data['abandonpos']
        rect_on = resources.pausemenu_data['abandon'].get_rect()
        rect_off = rect_on
        surf_on = resources.pausemenu_data['abandon']
        surf_off = resources.pausemenu_data['abandon']
        self.button1 = MenuButton(pos,rect_on,rect_off,surf_on,surf_off)
        
        pos = resources.pausemenu_data['yespos']
        rect_on = resources.pausemenu_data['yes_on'].get_rect()
        rect_off = rect_on
        surf_on = resources.pausemenu_data['yes_on']
        surf_off = resources.pausemenu_data['yes_off']
        self.button2 = MenuButton(pos,rect_on,rect_off,surf_on,surf_off)
        
        pos = resources.pausemenu_data['nopos']
        rect_on = resources.pausemenu_data['no_on'].get_rect()
        rect_off = rect_on
        surf_on = resources.pausemenu_data['no_on']
        surf_off = resources.pausemenu_data['no_off']
        self.button3 = MenuButton(pos,rect_on,rect_off,surf_on,surf_off)
        
        pos = resources.pausemenu_data['resetpos']
        rect_on = resources.pausemenu_data['reset_on'].get_rect()
        rect_off = rect_on
        surf_on = resources.pausemenu_data['reset_on']
        surf_off = resources.pausemenu_data['reset_off']
        self.button4 = MenuButton(pos,rect_on,rect_off,surf_on,surf_off)
        
        self.menulist = MenuList([self.button1,self.button2,self.button3,self.button4])
        self.menulist.SetGoFunc(0,self.GoFunc1)
        self.menulist.SetGoFunc(1,self.GoFunc2)
        self.menulist.SetGoFunc(2,self.GoFunc3)
        self.menulist.SetGoFunc(3,self.GoFunc4)
    
    def EscapeFunc(self,switchtoargs):
        self.director.change_scene('maingame', [False, switchtoargs])
    
    def GoFunc1(self, switchtoargs):
        pass
    
    def GoFunc2(self, switchtoargs):
        #self.director.change_scene(None, [])
        self.director.change_scene('titlescreen', [])
    
    def GoFunc3(self, switchtoargs):
        self.director.change_scene('maingame', [False, switchtoargs])
    
    def GoFunc4(self, switchtoargs):
        self.director.change_scene('maingame', [True, switchtoargs])

class TitleScreen(ClickOptionsScreen):    
    
    def __init__(self, director, window_size, fadelevel, savepath):
        super(TitleScreen, self).__init__(director,window_size,fadelevel)
        
        self.savepath = savepath
        
        pos = resources.titlescreen_data['newgamepos']
        rect_on = resources.titlescreen_data['newgame_on'].get_rect()
        rect_off = rect_on
        surf_on = resources.titlescreen_data['newgame_on']
        surf_off = resources.titlescreen_data['newgame_off']
        self.button1 = MenuButton(pos,rect_on,rect_off,surf_on,surf_off)
        
        pos = resources.titlescreen_data['continuepos']
        rect_on = resources.titlescreen_data['continue_on'].get_rect()
        rect_off = rect_on
        surf_on = resources.titlescreen_data['continue_on']
        surf_off = resources.titlescreen_data['continue_off']
        self.button2 = MenuButton(pos,rect_on,rect_off,surf_on,surf_off)
        
        self.starfield = MovingStarField()
    
    # Check music
    def on_switchto(self, switchtoargs):
        
        if os.path.isfile(self.savepath):
            self.menulist = MenuList([self.button1,self.button2])
            self.menulist.SetGoFunc(0,self.GoFunc1)
            self.menulist.SetGoFunc(1,self.GoFunc2)
        else:
            self.menulist = MenuList([self.button1])
            self.menulist.SetGoFunc(0,self.GoFunc1)
            
        super(TitleScreen, self).on_switchto(switchtoargs)
        pygame.mixer.music.stop()
        pygame.mixer.music.load(resources.musicpaths['intromusic002'])
        pygame.mixer.music.set_volume(1.0)
        pygame.mixer.music.play()
        self.timer = 0
    
    def EscapeFunc(self,switchtoargs):
        self.director.change_scene(None, [])
    
    def GoFunc1(self, switchtoargs):
        self.h_maingame.trackingdata.Reset()
        #self.director.change_scene('maingame', [True, resources.levellist[0]])
        self.director.change_scene('prologuescene1', resources.prologue1_data)
    
    def GoFunc2(self, switchtoargs):
        self.h_maingame.trackingdata.LoadProgressData()
        self.director.change_scene('maingame', [True, self.h_maingame.trackingdata.current_level])
    
    def on_update(self):
        self.starfield.Update()
        self.timer += 1
    
    def on_draw(self, screen):
        screen.blit(self.background, (0,0))
        self.starfield.Draw(screen)
        screen.blit(resources.titlescreen_data['titletext'], (0,0))
        if self.timer < 90:
            alpha = 0
        elif self.timer > 150:
            alpha = 255
        else:
            alpha = (255/60)*(self.timer-90)
        self.menulist.buttons[0].surf_on.set_alpha(alpha)
        self.menulist.buttons[0].surf_off.set_alpha(alpha)
        if len(self.menulist.buttons) > 1:
            self.menulist.buttons[1].surf_on.set_alpha(alpha)
            self.menulist.buttons[1].surf_off.set_alpha(alpha)
        self.menulist.Draw(screen)
        self.SetMousePos()
        tilecoords = resources.mouse_spritesheet_coords['hover'][resources.mouse_spriteseq['hover'][0]]
        screen.blit(resources.mouse_spritesheet, (self.cursor_pos_screen[0]-self.cursorsize/2,self.cursor_pos_screen[1]-self.cursorsize/2), area=tilecoords)
        

class SummaryScreen(ClickOptionsScreen):    
    
    def __init__(self, director, window_size, fadelevel):
        super(SummaryScreen, self).__init__(director,window_size,fadelevel)
        
        pos = (500,400)
        rect_on = resources.titlescreen_data['continue_on'].get_rect()
        rect_off = rect_on
        surf_on = resources.titlescreen_data['continue_on']
        surf_off = resources.titlescreen_data['continue_off']
        self.button1 = MenuButton(pos,rect_on,rect_off,surf_on,surf_off)
        
        self.menulist = MenuList([self.button1])
        self.menulist.SetGoFunc(0,self.GoFunc1)
        
        self.starfield = MovingStarField()
    
    def on_switchto(self, switchtoargs):
        super(SummaryScreen, self).on_switchto(switchtoargs)
        crewescapepod = self.switchtoargs[0][0]
        crewalive = self.switchtoargs[0][1]
        crewdead = self.switchtoargs[0][2]
        crewrescuedtotal = self.switchtoargs[0][3]
        self.next_level = self.switchtoargs[1]
        self.surf1 = resources.gamefont_green.RenderSentence("Crew Rescued: %d of %d"%(crewescapepod,crewalive), 160, 48, align='vcentre')
        self.surf2 = resources.gamefont_red.RenderSentence("Crew Dead: %d"%(crewdead), 160, 48, align='vcentre')
        self.surf3 = resources.gamefont_yellow.RenderSentence("Total Crew Rescued: %d"%(crewrescuedtotal), 160, 48, align='vcentre')
        
        pygame.mixer.music.stop()
        pygame.mixer.music.load(resources.musicpaths['herodown'])
        pygame.mixer.music.set_volume(1.0)
        pygame.mixer.music.play()
    
    def EscapeFunc(self,switchtoargs):
        self.director.change_scene(None, [])
    
    def GoFunc1(self, switchtoargs):
        if self.next_level == -1: # finished all levels
            self.director.change_scene('chapter5scene1', resources.chap5_1_data)
        elif "chapter" in self.next_level:
            if self.next_level == 'chapter2scene1':
                self.director.change_scene('chapter2scene1', resources.chap2_1_data)
            if self.next_level == 'chapter3scene1':
                self.director.change_scene('chapter3scene1', resources.chap3_1_data)
            if self.next_level == 'chapter4scene1':
                self.director.change_scene('chapter4scene1', resources.chap4_1_data)
            if self.next_level == 'chapter5scene1':
                self.director.change_scene('chapter5scene1', resources.chap5_1_data)
        else:
            self.director.change_scene('maingame', [True, self.next_level])
    
    def on_update(self):
        self.starfield.Update()
    
    def on_draw(self, screen):
        screen.blit(self.background, (0,0))
        self.starfield.Draw(screen)
        screen.blit(self.surf1,(240,150))
        screen.blit(self.surf2,(240,200))
        screen.blit(self.surf3,(240,250))
        self.menulist.Draw(screen)
        self.SetMousePos()
        tilecoords = resources.mouse_spritesheet_coords['hover'][resources.mouse_spriteseq['hover'][0]]
        screen.blit(resources.mouse_spritesheet, (self.cursor_pos_screen[0]-self.cursorsize/2,self.cursor_pos_screen[1]-self.cursorsize/2), area=tilecoords)

    
