#!/usr/bin/python

"""
level_editor.py - GUI for creating/editing level data
"""

import os
import pygame
from pygame.locals import *
from gamedirector import *

import resources_leveledit

# Start
def main(mainpath):

    # Initialise pygame
    pygame.init()
    pygame.mixer.init()
    
    # start up director
    framerate = 30
    #screen_res = (800,700)
    #screen_res = (1200,700)
    
    screen_res = (1200,700)
    #screen_res = (1380,700)
    
    window_title = "Level Editor: l/s (load/save), b (tile bg toggle), o (occupancy layer toggle), g (goal tile), f (fire layer), v (vac source layer), u (units), a (astronauts), x (switches), d (doors), p (vac pumps), k (key layer)"
    dir = GameDirector(window_title, screen_res, framerate)
    
    # Load resources
    resources_leveledit.init(mainpath)
    
    # Load edior
    level_edit = LevelEditor(dir, screen_res)
    dir.addscene('level_edit', level_edit)
    
    # start up director
    dir.change_scene('level_edit', [])
    dir.loop()

class LevelData(object):
    
    def __init__(self):
        pass
    
    def new_level(self, map_size):
        self.map_size = map_size[:]
        self.music = 'none'
        self.tilelayer = [-1 for i in xrange(map_size[0]*map_size[1])]
        #self.tilelayer_bg = [-1 for i in xrange(map_size[0]*map_size[1])]
        #self.tilelayer_fg = [-1 for i in xrange(map_size[0]*map_size[1])]
        self.occlayer = [0 for i in xrange(map_size[0]*map_size[1])]
        self.goallayer = [0 for i in xrange(map_size[0]*map_size[1])]
        self.firelayer = [0 for i in xrange(map_size[0]*map_size[1])]
        self.vacsourcelayer = ['null' for i in xrange(map_size[0]*map_size[1])]
        self.unitlayer = ['null' for i in xrange(map_size[0]*map_size[1])]
        self.astrolayer = ['null' for i in xrange(map_size[0]*map_size[1])]
        self.switches = []
        self.doors = []
        self.vacpumps = []
        self.keyslayer = [0 for i in xrange(map_size[0]*map_size[1])]
        self.escapepodtile = -1
        
    def AssignGrid(self,i,j,value):
        if i < 0 or i >= self.map_size[0] or j < 0 or j >= self.map_size[1]:
            return
        self.tilelayer[self.map_size[0]*j+i] = value
    
    """
    def AssignGridBG(self,i,j,value):
        if i < 0 or i >= self.map_size[0] or j < 0 or j >= self.map_size[1]:
            return
        self.tilelayer_bg[self.map_size[0]*j+i] = value
    
    def AssignGridFG(self,i,j,value):
        if i < 0 or i >= self.map_size[0] or j < 0 or j >= self.map_size[1]:
            return
        self.tilelayer_fg[self.map_size[0]*j+i] = value
    """
    
    def AssignOccGrid(self,i,j):
        if i < 0 or i >= self.map_size[0] or j < 0 or j >= self.map_size[1]:
            return
        self.occlayer[self.map_size[0]*j+i] = (self.occlayer[self.map_size[0]*j+i]+1) % 2
    
    def AssignGoalGrid(self,i,j):
        if i < 0 or i >= self.map_size[0] or j < 0 or j >= self.map_size[1]:
            return
        self.goallayer[self.map_size[0]*j+i] = (self.goallayer[self.map_size[0]*j+i]+1) % 2
    
    def AssignFireGrid(self,i,j):
        if i < 0 or i >= self.map_size[0] or j < 0 or j >= self.map_size[1]:
            return
        self.firelayer[self.map_size[0]*j+i] = (self.firelayer[self.map_size[0]*j+i]+1) % 2
    
    def AssignVacsourceGrid(self,i,j):
        if i < 0 or i >= self.map_size[0] or j < 0 or j >= self.map_size[1]:
            return
        if self.vacsourcelayer[self.map_size[0]*j+i] == 'null':
            self.vacsourcelayer[self.map_size[0]*j+i] = [10]
        else:
            self.vacsourcelayer[self.map_size[0]*j+i] = 'null'
    
    def AssignUnitGrid(self,i,j,type):
        if i < 0 or i >= self.map_size[0] or j < 0 or j >= self.map_size[1]:
            return
        if self.unitlayer[self.map_size[0]*j+i] == 'null':
            self.unitlayer[self.map_size[0]*j+i] = [type,0]
        else:
            self.unitlayer[self.map_size[0]*j+i] = 'null'
    
    def AssignKeyGrid(self,i,j):
        if i < 0 or i >= self.map_size[0] or j < 0 or j >= self.map_size[1]:
            return
        self.keyslayer[self.map_size[0]*j+i] = (self.keyslayer[self.map_size[0]*j+i]+1) % 2
    
    def AssignEscapePod(self,i,j):
        if i < 0 or i >= self.map_size[0] or j < 0 or j >= self.map_size[1]:
            return
        self.escapepodtile = self.map_size[0]*j+i
    
    def save_level(self, filepath):
        f = open(filepath, "w");
        f.write('%d %d\n'%(self.map_size[0],self.map_size[1])) # map size
        f.write('%s\n'%(self.music)) # music id
        for j in xrange(self.map_size[1]): # tile graphics layer
            for i in xrange(self.map_size[0]):
                f.write('%d '%(self.tilelayer[self.map_size[0]*j+i]))
            f.write('\n')
        """
        for j in xrange(self.map_size[1]):
            for i in xrange(self.map_size[0]):
                f.write('%d '%(self.tilelayer_bg[self.map_size[0]*j+i]))
            f.write('\n')
        for j in xrange(self.map_size[1]):
            for i in xrange(self.map_size[0]):
                f.write('%d '%(self.tilelayer_fg[self.map_size[0]*j+i]))
            f.write('\n')
        """
        for j in xrange(self.map_size[1]): # occupancy grid layer
            for i in xrange(self.map_size[0]):
                f.write('%d '%(self.occlayer[self.map_size[0]*j+i]))
            f.write('\n')
        """
        for j in xrange(self.map_size[1]): # goal tile locations (escape pod)
            for i in xrange(self.map_size[0]):
                if self.goallayer[self.map_size[0]*j+i] == 1:
                    f.write('%d '%(self.map_size[0]*j+i))
        f.write('\n')
        """
        f.write('%d\n'%(self.escapepodtile))
        
        for j in xrange(self.map_size[1]): # fire tile locations
            for i in xrange(self.map_size[0]):
                if self.firelayer[self.map_size[0]*j+i] == 1:
                    f.write('%d '%(self.map_size[0]*j+i))
        f.write('\n')
        
        for j in xrange(self.map_size[1]): # key tile locations
            for i in xrange(self.map_size[0]):
                if self.keyslayer[self.map_size[0]*j+i] == 1:
                    f.write('%d '%(self.map_size[0]*j+i))
        f.write('\n')
        
        numvacsource = sum([not i == 'null' for i in self.vacsourcelayer]) # vac sources
        f.write('%d\n'%(numvacsource))
        for j in xrange(self.map_size[1]):
            for i in xrange(self.map_size[0]):
                if not self.vacsourcelayer[self.map_size[0]*j+i] == 'null':
                    f.write('%d %d\n'%(self.map_size[0]*j+i,self.vacsourcelayer[self.map_size[0]*j+i][0]))
        
        numunits = sum([not i == 'null' for i in self.unitlayer]) # units
        f.write('%d\n'%(numunits))
        for j in xrange(self.map_size[1]):
            for i in xrange(self.map_size[0]):
                if not self.unitlayer[self.map_size[0]*j+i] == 'null':
                    f.write('%d %s %d\n'%(self.map_size[0]*j+i,self.unitlayer[self.map_size[0]*j+i][0],self.unitlayer[self.map_size[0]*j+i][1]))
        
        f.write('%d\n'%(len(self.switches))) # switches (tile, stand tile, type, mode, assoc id list)
        for i in xrange(len(self.switches)):
            f.write('%d %d %s %s %d '%(self.switches[i][0],self.switches[i][1],self.switches[i][2],\
                self.switches[i][3],self.switches[i][4]))
            for j in xrange(len(self.switches[i])-5):
                f.write('%d '%(self.switches[i][j+5]))
            f.write('\n')
        
        f.write('%d\n'%(len(self.doors))) # doors (tile1, tile2, state)
        for i in xrange(len(self.doors)):
            f.write('%d %d %d\n'%(self.doors[i][0],self.doors[i][1],self.doors[i][2]))
        
        f.write('%d\n'%(len(self.vacpumps))) # vacuum pumps (tile, air tile, radius, power)
        for i in xrange(len(self.vacpumps)):
            f.write('%d %d %d\n'%(self.vacpumps[i][0],self.vacpumps[i][1],self.vacpumps[i][2]))
        
        f.close()
    
    def load_level(self, filepath):
        f = open(filepath, "r")
        self.map_size = [int(i) for i in f.readline().split()]
        self.music = f.readline().split()[0]
        self.tilelayer = []
        for j in xrange(self.map_size[1]):
            self.tilelayer.extend([int(i) for i in f.readline().split()])
        #self.tilelayer_bg = []
        #for j in xrange(self.map_size[1]):
        #    self.tilelayer_bg.extend([int(i) for i in f.readline().split()])
        #self.tilelayer_fg = []
        #for j in xrange(self.map_size[1]):
        #    self.tilelayer_fg.extend([int(i) for i in f.readline().split()])
        self.occlayer = []
        for j in xrange(self.map_size[1]):
            self.occlayer.extend([int(i) for i in f.readline().split()])
        self.escapepodtile = [int(i) for i in f.readline().split()][0]
        """
        tiles = [int(i) for i in f.readline().split()]
        self.goallayer = [0 for i in xrange(self.map_size[0]*self.map_size[1])]
        for t in tiles:
            self.goallayer[t] = 1
        """
        tiles = [int(i) for i in f.readline().split()]
        self.firelayer = [0 for i in xrange(self.map_size[0]*self.map_size[1])]
        for t in tiles:
            self.firelayer[t] = 1
        tiles = [int(i) for i in f.readline().split()]
        self.keyslayer = [0 for i in xrange(self.map_size[0]*self.map_size[1])]
        for t in tiles:
            self.keyslayer[t] = 1
        self.vacsourcelayer = ['null' for i in xrange(self.map_size[0]*self.map_size[1])]
        numitems = [int(i) for i in f.readline().split()][0]
        for j in xrange(numitems):
            data = f.readline().split()
            self.vacsourcelayer[int(data[0])] = [int(data[1])]
        self.unitlayer = ['null' for i in xrange(self.map_size[0]*self.map_size[1])]
        numitems = [int(i) for i in f.readline().split()][0]
        for j in xrange(numitems):
            data = f.readline().split()
            self.unitlayer[int(data[0])] = [data[1], int(data[2])]
        self.switches = []
        num = [int(i) for i in f.readline().split()][0]
        for i in xrange(num):
            data = f.readline().split()
            self.switches.append([int(data[0]), int(data[1]), data[2], data[3], int(data[4])])
            for j in xrange(len(data)-5):
                self.switches[-1].append(int(data[j+5]))
        self.doors = []
        num = [int(i) for i in f.readline().split()][0]
        for i in xrange(num):
            data = f.readline().split()
            self.doors.append([int(data[0]), int(data[1]), int(data[2])])
        self.vacpumps = []
        num = [int(i) for i in f.readline().split()][0]
        for i in xrange(num):
            data = f.readline().split()
            self.vacpumps.append([int(data[0]), int(data[1]), int(data[2])])
        f.close()

class LevelEditor(GameScene):
    def __init__(self, director, window_size):
        super(LevelEditor, self).__init__(director)
        
        self.window_size = window_size
        self.background = pygame.Surface(window_size)
        self.background.fill((0, 0, 0))
        self.background.convert()
        
        self.mouse_pos = (0,0)
        self.draw_type = -1
        self.draw_type_item = 'null'
        self.tilemode = 0
        #self.tilebgmode = 0
        #self.tilefgmode = 0
        self.objectdrawtype = 'none'
        self.map_size_default = (50,30)
        self.tilesize = 16
        
        self.level = LevelData()
        self.level.new_level(self.map_size_default)
        
        self.npal_x = 8
        self.npal_y = 36
        
        self.palette = pygame.Surface((self.npal_x*self.tilesize, self.npal_y*self.tilesize))
        self.palette.fill((0, 0, 0))
        self.palette.convert()
        
        for j in xrange(self.npal_y):
            for i in xrange(self.npal_x):
                self.palette.blit(resources_leveledit.tiles, (i*16,j*16), area=resources_leveledit.tiles_coords[i+self.npal_x*j])
        
        self.reset_offsets()
    
    def reset_offsets(self):
        self.draw_offset = (16, self.window_size[1]/2 - (self.level.map_size[1]*self.tilesize)/2)
        self.draw_offset_palette = (self.window_size[0]-16-self.npal_x*self.tilesize, 2*self.tilesize)
        self.switchuserdata = []
        self.dooruserdata = []
        self.pumpuserdata = []
    
    def on_switchto(self, switchtoargs):
        pass
    
    def on_update(self):
        self.mouse_pos = pygame.mouse.get_pos()
        #print self.mouse_pos
        
    def on_event(self, events):
        for event in events:
            if event.type == KEYDOWN and event.key == K_DOWN:
                self.draw_type = self.draw_type + 1
                if self.draw_type == self.npal_x*self.npal_y:
                    self.draw_type = -1
            if event.type == KEYDOWN and event.key == K_UP:
                self.draw_type = self.draw_type - 1
                if self.draw_type == -2:
                    self.draw_type = self.npal_x*self.npal_y-1
            if event.type == KEYDOWN and event.key == K_o:
                self.tilemode = (self.tilemode+1) % 2
            #if event.type == KEYDOWN and event.key == K_b:
            #    self.tilebgmode = (self.tilebgmode+1) % 2
            #if event.type == KEYDOWN and event.key == K_f:
            #    self.tilefgmode = (self.tilefgmode+1) % 2
            if event.type == KEYDOWN and event.key == K_q:
                self.objectdrawtype = 'none'
                self.switchuserdata = []
                self.dooruserdata = []
                self.pumpuserdata = []
            if event.type == KEYDOWN and event.key == K_g:
                self.objectdrawtype = 'goal'
            if event.type == KEYDOWN and event.key == K_f:
                self.objectdrawtype = 'fire'
            if event.type == KEYDOWN and event.key == K_v:
                self.objectdrawtype = 'vacsource'
            if event.type == KEYDOWN and event.key == K_u:
                self.objectdrawtype = 'unit'
            if event.type == KEYDOWN and event.key == K_a:
                self.objectdrawtype = 'astro'
            if event.type == KEYDOWN and event.key == K_x:
                self.objectdrawtype = 'switches'
            if event.type == KEYDOWN and event.key == K_d:
                self.objectdrawtype = 'door'
            if event.type == KEYDOWN and event.key == K_p:
                self.objectdrawtype = 'pump'
            if event.type == KEYDOWN and event.key == K_k:
                self.objectdrawtype = 'key'
            if event.type == KEYDOWN and event.key == K_e:
                self.objectdrawtype = 'escapepod'
            if event.type == KEYDOWN and event.key == K_s:
                self.level.save_level("Data/level_twosections.txt")
            if event.type == KEYDOWN and event.key == K_l:
                self.level.load_level("Data/level_twosections.txt")
                self.reset_offsets()
            if event.type == KEYDOWN and event.key == K_n:
                self.level.new_level(self.map_size_default)
                self.reset_offsets()
            elif event.type == MOUSEBUTTONDOWN:
                if (self.mouse_pos[0] < (self.draw_offset[0]+self.level.map_size[0]*self.tilesize)):
                    coords = ((self.mouse_pos[0]-self.draw_offset[0])/self.tilesize, (self.mouse_pos[1]-self.draw_offset[1])/self.tilesize)
                    tile = self.level.map_size[0]*coords[1]+coords[0]
                    if coords[0] >= 0 and coords[0] < self.level.map_size[0] and coords[1] >= 0 and coords[1] < self.level.map_size[1]:
                        if self.objectdrawtype == 'none':
                            if self.tilemode == 1:
                                self.level.AssignOccGrid(coords[0],coords[1])
                            elif self.tilemode == 0:
                                self.level.AssignGrid(coords[0],coords[1],self.draw_type)
                                #if self.tilebgmode == 1:
                                #    self.level.AssignGridBG(coords[0],coords[1],self.draw_type)
                                #elif self.tilefgmode == 1:
                                #    self.level.AssignGridFG(coords[0],coords[1],self.draw_type)
                                #else:
                                #    self.level.AssignGrid(coords[0],coords[1],self.draw_type)
                        elif self.objectdrawtype == 'goal':
                            self.level.AssignGoalGrid(coords[0],coords[1])
                        elif self.objectdrawtype == 'fire':
                            self.level.AssignFireGrid(coords[0],coords[1])
                        elif self.objectdrawtype == 'key':
                            self.level.AssignKeyGrid(coords[0],coords[1])
                        elif self.objectdrawtype == 'vacsource':
                            self.level.AssignVacsourceGrid(coords[0],coords[1])
                        elif self.objectdrawtype == 'unit':
                            self.level.AssignUnitGrid(coords[0],coords[1],'unit')
                        elif self.objectdrawtype == 'astro':
                            self.level.AssignUnitGrid(coords[0],coords[1],'astro')
                        elif self.objectdrawtype == 'switches':
                            if len(self.switchuserdata) < 2:
                                self.switchuserdata.append(tile)
                            else:
                                type = 'none'
                                id = -1
                                for i in xrange(len(self.level.doors)):
                                    if self.level.doors[i][0] == tile:
                                        type = 'door'
                                        id = i
                                for i in xrange(len(self.level.vacpumps)):
                                    if self.level.vacpumps[i][0] == tile:
                                        type = 'pump'
                                        id = i
                                for i in xrange(len(self.level.switches)):
                                    if self.level.switches[i][0] == tile:
                                        type = 'powerplug'
                                        id = i
                                if type == 'powerplug':
                                    self.level.switches.append([self.switchuserdata[0],self.switchuserdata[1],type,'once',1,id])
                                else:
                                    self.level.switches.append([self.switchuserdata[0],self.switchuserdata[1],type,'normal',1,id])
                                self.switchuserdata = []
                        elif self.objectdrawtype == 'door':
                            if len(self.dooruserdata) < 1:
                                self.dooruserdata.append(tile)
                            else:
                                self.level.doors.append([self.dooruserdata[0],tile,0])
                                self.dooruserdata = []
                        elif self.objectdrawtype == 'pump':
                            if len(self.pumpuserdata) < 1:
                                self.pumpuserdata.append(tile)
                            else:
                                self.level.vacpumps.append([self.pumpuserdata[0],tile,10])
                                self.pumpuserdata = []
                        elif self.objectdrawtype == 'escapepod':
                            self.level.AssignEscapePod(coords[0],coords[1])
                elif (self.mouse_pos[0] > self.draw_offset_palette[0] and self.mouse_pos[1] < (self.draw_offset_palette[1]+self.npal_y*self.tilesize)):
                    coords = ((self.mouse_pos[0]-self.draw_offset_palette[0])/self.tilesize, (self.mouse_pos[1]-self.draw_offset_palette[1])/self.tilesize)
                    if coords[0] >= 0 and coords[0] < self.npal_x and coords[1] >= 0 and coords[1] < self.npal_y:
                        self.draw_type = coords[0]+self.npal_x*coords[1]
    
    def on_draw(self, screen):
        
        ts = self.tilesize
        msx = self.level.map_size[0]
        
        screen.blit(self.background, (0, 0))
        pygame.draw.rect(screen,(255,255,255), \
            pygame.Rect((self.draw_offset[0]-1,self.draw_offset[1]-1),(self.level.map_size[0]*self.tilesize+2,self.level.map_size[1]*self.tilesize+2)),1)
        
        pygame.draw.rect(screen,(128,128,255), \
            pygame.Rect((self.draw_offset[0],self.draw_offset[1]),(self.level.map_size[0]*self.tilesize,self.level.map_size[1]*self.tilesize)))
        
        for i in range(self.level.map_size[0]):
            for j in range(self.level.map_size[1]):
                coords = [self.draw_offset[0]+i*self.tilesize, self.draw_offset[1]+j*self.tilesize]
                
                if self.tilemode == 1:
                    occval = self.level.occlayer[self.level.map_size[0]*j+i]
                    if occval == 0:
                        pygame.draw.rect(screen, (0,0,0), pygame.Rect((coords[0],coords[1]),(self.tilesize,self.tilesize)))
                    elif occval == 1:
                        pygame.draw.rect(screen, (255,255,255), pygame.Rect((coords[0],coords[1]),(self.tilesize,self.tilesize)))
                else:
                    tileval = self.level.tilelayer[self.level.map_size[0]*j+i]
                    if tileval >= 0:
                        tilecoords = resources_leveledit.tiles_coords[tileval]
                        screen.blit(resources_leveledit.tiles, (coords[0],coords[1]), area=tilecoords)
                    #tilevalbg = self.level.tilelayer_bg[self.level.map_size[0]*j+i]
                    #tilevalfg = self.level.tilelayer_fg[self.level.map_size[0]*j+i]
                    #if tilevalbg >= 0:
                    #    tilecoords = resources.tiles_coords[tilevalbg]
                    #    screen.blit(resources.tiles, (coords[0],coords[1]), area=tilecoords)
                    #if tileval >= 0 and self.tilebgmode == 0:
                    #    tilecoords = resources.tiles_coords[tileval]
                    #    screen.blit(resources.tiles, (coords[0],coords[1]), area=tilecoords)
                    #if tilevalfg >= 0 and self.tilebgmode == 0 and self.tilefgmode == 1:
                    #    tilecoords = resources.tiles_coords[tilevalfg]
                    #    screen.blit(resources.tiles, (coords[0],coords[1]), area=tilecoords)
        
        # draw escape pod
        if self.level.escapepodtile >= 0 and not self.tilemode == 1:
            i = self.level.escapepodtile%msx
            j = self.level.escapepodtile/msx - 4
            coords = [self.draw_offset[0]+i*self.tilesize, self.draw_offset[1]+j*self.tilesize]
            tilecoords = list(resources_leveledit.tiles_coords[216])
            tilecoords[2] = 7*self.tilesize
            tilecoords[3] = 9*self.tilesize
            tilecoords = tuple(tilecoords)
            screen.blit(resources_leveledit.tiles, (coords[0],coords[1]), area=tilecoords)
        
        screen.blit(self.palette, self.draw_offset_palette)
        if self.draw_type >= 0:
            pygame.draw.rect(screen,(255,0,0), \
                pygame.Rect((self.draw_offset_palette[0]+16*(self.draw_type%self.npal_x),self.draw_offset_palette[1]+16*(self.draw_type/self.npal_x)),(16,16)),1)
        
        if self.objectdrawtype == 'goal':
            for i in range(self.level.map_size[0]):
                for j in range(self.level.map_size[1]):
                    if self.level.goallayer[self.level.map_size[0]*j+i] == 1:
                        coords = [self.draw_offset[0]+i*self.tilesize, self.draw_offset[1]+j*self.tilesize]
                        pygame.draw.rect(screen,(0,255,0),pygame.Rect((coords[0],coords[1]),(ts,ts)),0)
        elif self.objectdrawtype == 'fire':
            for i in range(self.level.map_size[0]):
                for j in range(self.level.map_size[1]):
                    if self.level.firelayer[self.level.map_size[0]*j+i] == 1:
                        coords = [self.draw_offset[0]+i*self.tilesize, self.draw_offset[1]+j*self.tilesize]
                        pygame.draw.rect(screen,(255,0,0),pygame.Rect((coords[0],coords[1]),(ts,ts)),0)
        elif self.objectdrawtype == 'key':
            for i in range(self.level.map_size[0]):
                for j in range(self.level.map_size[1]):
                    if self.level.keyslayer[self.level.map_size[0]*j+i] == 1:
                        coords = [self.draw_offset[0]+i*self.tilesize, self.draw_offset[1]+j*self.tilesize]
                        pygame.draw.rect(screen,(255,255,0),pygame.Rect((coords[0],coords[1]),(ts,ts)),0)
        elif self.objectdrawtype == 'vacsource':
            for i in range(self.level.map_size[0]):
                for j in range(self.level.map_size[1]):
                    if not self.level.vacsourcelayer[self.level.map_size[0]*j+i] == 'null':
                        coords = [self.draw_offset[0]+i*self.tilesize, self.draw_offset[1]+j*self.tilesize]
                        pygame.draw.rect(screen,(0,0,255),pygame.Rect((coords[0],coords[1]),(ts,ts)),0)
        elif self.objectdrawtype == 'unit':
            for i in range(self.level.map_size[0]):
                for j in range(self.level.map_size[1]):
                    if not self.level.unitlayer[self.level.map_size[0]*j+i] == 'null':
                        if self.level.unitlayer[self.level.map_size[0]*j+i][0] == 'unit':
                            coords = [self.draw_offset[0]+i*self.tilesize, self.draw_offset[1]+j*self.tilesize]
                            pygame.draw.rect(screen,(0,255,0),pygame.Rect((coords[0],coords[1]),(ts,ts)),0)
        elif self.objectdrawtype == 'astro':
            for i in range(self.level.map_size[0]):
                for j in range(self.level.map_size[1]):
                    if not self.level.unitlayer[self.level.map_size[0]*j+i] == 'null':
                        if self.level.unitlayer[self.level.map_size[0]*j+i][0] == 'astro':
                            coords = [self.draw_offset[0]+i*self.tilesize, self.draw_offset[1]+j*self.tilesize]
                            pygame.draw.rect(screen,(0,255,0),pygame.Rect((coords[0],coords[1]),(ts,ts)),0)
        elif self.objectdrawtype == 'switches':
            for s in self.level.switches:
                tile1 = s[0]
                tile2 = s[1]
                type = s[2]
                ids = s[5:]
                coords1 = [self.draw_offset[0]+(tile1%msx)*self.tilesize, self.draw_offset[1]+(tile1/msx)*self.tilesize]
                pygame.draw.rect(screen,(255,255,255),pygame.Rect((coords1[0],coords1[1]),(ts,ts)),0)
                coords2 = [self.draw_offset[0]+(tile2%msx)*self.tilesize, self.draw_offset[1]+(tile2/msx)*self.tilesize]
                pygame.draw.rect(screen,(0,0,255),pygame.Rect((coords2[0],coords2[1]),(ts,ts)),0)
                if type == 'door':
                    for id in ids:
                        tile = self.level.doors[id][0]
                        coords = [self.draw_offset[0]+(tile%msx)*self.tilesize, self.draw_offset[1]+(tile/msx)*self.tilesize]
                        pygame.draw.rect(screen,(255,0,0),pygame.Rect((coords[0],coords[1]),(ts,ts)),1)
                elif type == 'pump':
                    for id in ids:
                        tile = self.level.vacpumps[id][0]
                        coords = [self.draw_offset[0]+(tile%msx)*self.tilesize, self.draw_offset[1]+(tile/msx)*self.tilesize]
                        pygame.draw.rect(screen,(255,0,0),pygame.Rect((coords[0],coords[1]),(ts,ts)),1)
                elif type == 'powerplug':
                    for id in ids:
                        tile = self.level.switches[id][0]
                        coords = [self.draw_offset[0]+(tile%msx)*self.tilesize, self.draw_offset[1]+(tile/msx)*self.tilesize]
                        pygame.draw.rect(screen,(255,0,0),pygame.Rect((coords[0],coords[1]),(ts,ts)),1)
        elif self.objectdrawtype == 'door':
            for s in self.level.doors:
                tile1 = s[0]
                tile2 = s[1]
                coords1 = [self.draw_offset[0]+(tile1%msx)*self.tilesize, self.draw_offset[1]+(tile1/msx)*self.tilesize]
                pygame.draw.rect(screen,(0,0,255),pygame.Rect((coords1[0],coords1[1]),(ts,ts)),0)
                coords2 = [self.draw_offset[0]+(tile2%msx)*self.tilesize, self.draw_offset[1]+(tile2/msx)*self.tilesize]
                pygame.draw.rect(screen,(0,0,255),pygame.Rect((coords2[0],coords2[1]),(ts,ts)),1)
        elif self.objectdrawtype == 'pump':
            for s in self.level.vacpumps:
                tile1 = s[0]
                tile2 = s[1]
                coords1 = [self.draw_offset[0]+(tile1%msx)*self.tilesize, self.draw_offset[1]+(tile1/msx)*self.tilesize]
                pygame.draw.rect(screen,(0,0,255),pygame.Rect((coords1[0],coords1[1]),(ts,ts)),0)
                coords2 = [self.draw_offset[0]+(tile2%msx)*self.tilesize, self.draw_offset[1]+(tile2/msx)*self.tilesize]
                pygame.draw.rect(screen,(255,255,255),pygame.Rect((coords2[0],coords2[1]),(ts,ts)),1)
        if self.objectdrawtype == 'escapepod':
            if self.level.escapepodtile >= 0:
                i = self.level.escapepodtile%msx
                j = self.level.escapepodtile/msx
                coords = [self.draw_offset[0]+i*self.tilesize, self.draw_offset[1]+j*self.tilesize]
                pygame.draw.rect(screen,(255,0,0),pygame.Rect((coords[0],coords[1]),(ts,ts)),0)
        

