#!/usr/bin/python

"""
main.py - Main entry point for game
"""

import os
import pygame
from pygame.locals import *
from gamedirector import *

import resources
import game
from common import CutScene, CutSceneShipExplode

# Start
def main(mainpath):

    # Initialise pygame
    pygame.init()
    pygame.mixer.init()
    pygame.mouse.set_visible(False)
    
    # start up director
    framerate = 30
    screen_res = (640,480)
    window_title = "Game"
    dir = GameDirector(window_title, screen_res, framerate)
    
    # Load resources
    resources.init(mainpath)
    
    # Load game scenes
    logocutscene = CutScene(dir, screen_res)
    dir.addscene('logocutscene', logocutscene)
    
    quotescene = CutScene(dir, screen_res)
    dir.addscene('quotescene', quotescene)
    
    savepath = os.path.join(mainpath,'data','saved_progress.txt')
    titlescreen = game.TitleScreen(dir, screen_res, 0, savepath)
    dir.addscene('titlescreen', titlescreen)
    
    pausegame = game.PauseGame(dir, screen_res, 128)
    dir.addscene('pausegame', pausegame)
    
    maingame = game.MainGame(dir, screen_res)
    maingame.h_pausescene = pausegame
    dir.addscene('maingame', maingame)
    maingame.trackingdata.savepath = os.path.join(mainpath,'data','saved_progress.txt')
    titlescreen.h_maingame = maingame
    
    summaryscreen = game.SummaryScreen(dir, screen_res, 0)
    dir.addscene('summaryscreen', summaryscreen)
    
    # major cut scenes:
    
    # prologue:
    prologuescene1 = CutScene(dir, screen_res)
    dir.addscene('prologuescene1', prologuescene1)
    prologuescene2 = CutScene(dir, screen_res)
    dir.addscene('prologuescene2', prologuescene2)
    prologuescene3 = CutScene(dir, screen_res)
    dir.addscene('prologuescene3', prologuescene3)
    prologuescene4 = CutScene(dir, screen_res)
    dir.addscene('prologuescene4', prologuescene4)
    
    chapter1scene1 = CutScene(dir, screen_res)
    dir.addscene('chapter1scene1', chapter1scene1)
    chapter1scene2 = CutScene(dir, screen_res)
    dir.addscene('chapter1scene2', chapter1scene2)
    chapter1scene3 = CutSceneShipExplode(dir, screen_res)
    dir.addscene('chapter1scene3', chapter1scene3)
    
    chapter2scene1 = CutScene(dir, screen_res)
    dir.addscene('chapter2scene1', chapter2scene1)
    chapter2scene2 = CutScene(dir, screen_res)
    dir.addscene('chapter2scene2', chapter2scene2)
    chapter2scene3 = CutScene(dir, screen_res)
    dir.addscene('chapter2scene3', chapter2scene3)
    chapter2scene4 = CutScene(dir, screen_res)
    dir.addscene('chapter2scene4', chapter2scene4)
    chapter2scene5 = CutScene(dir, screen_res)
    dir.addscene('chapter2scene5', chapter2scene5)
    chapter2scene6 = CutScene(dir, screen_res)
    dir.addscene('chapter2scene6', chapter2scene6)
    chapter2scene7 = CutScene(dir, screen_res)
    dir.addscene('chapter2scene7', chapter2scene7)
    chapter2scene8 = CutScene(dir, screen_res)
    dir.addscene('chapter2scene8', chapter2scene8)
    
    chapter3scene1 = CutScene(dir, screen_res)
    dir.addscene('chapter3scene1', chapter3scene1)
    chapter3scene2 = CutScene(dir, screen_res)
    dir.addscene('chapter3scene2', chapter3scene2)
    chapter3scene3 = CutScene(dir, screen_res)
    dir.addscene('chapter3scene3', chapter3scene3)
    chapter3scene4 = CutScene(dir, screen_res)
    dir.addscene('chapter3scene4', chapter3scene4)
    
    chapter4scene1 = CutScene(dir, screen_res)
    dir.addscene('chapter4scene1', chapter4scene1)
    chapter4scene2 = CutScene(dir, screen_res)
    dir.addscene('chapter4scene2', chapter4scene2)
    chapter4scene3 = CutScene(dir, screen_res)
    dir.addscene('chapter4scene3', chapter4scene3)
    
    chapter5scene1 = CutScene(dir, screen_res)
    dir.addscene('chapter5scene1', chapter5scene1)
    chapter5scene2 = CutScene(dir, screen_res)
    dir.addscene('chapter5scene2', chapter5scene2)
    chapter5scene3 = CutScene(dir, screen_res)
    dir.addscene('chapter5scene3', chapter5scene3)
    
    # start up director
    dir.change_scene('logocutscene', resources.logocutscene_data)
    #dir.change_scene('chapter3scene1', resources.chap3_1_data)
    #dir.change_scene('titlescreen', [])
    #dir.change_scene('maingame', [True, 'level_twosections'])
    #dir.change_scene('summaryscreen', [[5,7,1,20], 'level_default2'])
    dir.loop()
    
    #print "Avg. framerate: %f"%(maingame.avgframerate)


