#!/usr/bin/python

"""
path.py - objects for path following entities, other ccupancy space calculations
"""

import os
import pygame
from pygame.locals import *
from gamedirector import *

import resources
from math import *
import heapq
import Queue

class Planner(object):
    def __init__(self,mapsize,tilesize,occlayer):
        self.maxsteps = 600
        (self.w, self.h) = mapsize
        self.tilesize = tilesize
        self.h_occlayer = occlayer
        self.pn = 0
        self.touched = [0 for i in xrange(self.w*self.h)]
        self.cost_so_far = [0 for i in xrange(self.w*self.h)]
        self.came_from = [0 for i in xrange(self.w*self.h)]
    
    def getneighbours(self, i):
        neighbours = []
        x = i%self.w
        y = i/self.w
        cost = 1
        costd = sqrt(2)
        if x > 0 and not self.h_occlayer[i-1]:
            neighbours.append((i-1,cost))
            if y > 0 and not self.h_occlayer[i-self.w] and not self.h_occlayer[i-self.w-1]:
                neighbours.append((i-self.w-1,costd))
            if y < (self.h-1) and not self.h_occlayer[i+self.w] and not self.h_occlayer[i+self.w-1]:
                neighbours.append((i+self.w-1,costd))
        if x < (self.w-1) and not self.h_occlayer[i+1]:
            neighbours.append((i+1,cost))
            if y > 0 and not self.h_occlayer[i-self.w] and not self.h_occlayer[i-self.w+1]:
                neighbours.append((i-self.w+1,costd))
            if y < (self.h-1) and not self.h_occlayer[i+self.w] and not self.h_occlayer[i+self.w+1]:
                neighbours.append((i+self.w+1,costd))
        if y > 0 and not self.h_occlayer[i-self.w]:
            neighbours.append((i-self.w,cost))
        if y < (self.h-1) and not self.h_occlayer[i+self.w]:
            neighbours.append((i+self.w,cost))
        return neighbours
    
    def heuristic(self,i,j):
        xi = i%self.w
        yi = i/self.w
        xj = j%self.w
        yj = j/self.w
        return abs(xi - xj) + abs(yi - yj)
    
    def astar_path(self, start, goal):
        if self.h_occlayer[goal]: # goal is an obstacle
            return []
        self.pn += 1
        frontier = []
        heapq.heappush(frontier, (0, start))
        self.touched[start] = self.pn
        self.cost_so_far[start] = 0
        self.came_from[start] = None
        
        nsteps = 0
        
        while True:
            nsteps += 1
            if len(frontier) == 0: # ran out of options, no path
                #return None
                return []
            current = heapq.heappop(frontier)[1]
            
            if nsteps == self.maxsteps: # tried too hard
                return []
            
            if current == goal:  # got it
                path = [current]
                while not self.came_from[current] == None: # step backwards through came froms to get path
                    current = self.came_from[current]
                    path.insert(0,current)
                return path
            
            for next in self.getneighbours(current):
                new_cost = self.cost_so_far[current] + next[1]
                if self.touched[next[0]] < self.pn or new_cost < self.cost_so_far[next[0]]:
                    self.touched[next[0]] = self.pn
                    self.cost_so_far[next[0]] = new_cost
                    priority = new_cost + self.heuristic(goal, next[0])
                    heapq.heappush(frontier, (priority, next[0]))
                    self.came_from[next[0]] = current
    
class VacuumController(Planner):
    def __init__(self,mapsize,tilesize,occlayer,airlayer,vacsources):
        super(VacuumController, self).__init__(mapsize,tilesize,occlayer)
        self.h_airlayer = airlayer
        self.h_vacsources = vacsources
    
    def UpdateVacuumCells(self,start,val,max_volume):
        self.pn += 1
        frontier = Queue.Queue()
        self.touched[start] = self.pn
        self.h_airlayer[start] = val
        frontier.put(start)
        volume = 0
        while not frontier.empty():
            current = frontier.get()
            for next in self.getneighbours(current):
                if self.touched[next[0]] < self.pn:
                    volume += 1
                    if max_volume > 0 and volume > max_volume:
                        return False
                    self.touched[next[0]] = self.pn
                    frontier.put(next[0])
                    self.h_airlayer[next[0]] = val
        return True
    
    """
    def UpdateVacuum(self,doortiles):
        for vs in self.h_vacsources:
            self.UpdateVacuumCells(vs[0],0,-1)
        for dt in doortiles:
            neighbours = self.getneighbours(dt)
            for n in neighbours:
                if self.h_airlayer[n[0]] == 0:
                    self.UpdateVacuumCells(dt,0,-1)
    """
    def UpdateVacuum(self,doortiles): # new vacuum function: re-fills air when no long exposed to space
        for i in xrange(len(self.h_airlayer)):
            self.h_airlayer[i] = 1
        for vs in self.h_vacsources:
            self.UpdateVacuumCells(vs[0],0,-1)

    def PumpAir(self,tile,val,maxvolume):
        if val == 0: # de-pressurising
            self.UpdateVacuumCells(tile,0,-1)
            return True
        else: # attempting to pump air
            success = self.UpdateVacuumCells(tile,1,maxvolume) # try to pump air and see
            if success:
                return True
            else:
                self.UpdateVacuumCells(tile,0,-1) # re-pump vacuum and alert not enought air
                return False