#!/usr/bin/python

"""
resources.py - Load all resource data once 
"""

import os, sys
import pygame
from pygame.locals import *
import random

class InitLevelData(object):
    def __init__(self, filepath):
        
        f = open(filepath, "r")
        self.map_size = [int(i) for i in f.readline().split()]
        self.music = f.readline().split()[0]
        self.tilelayer = []
        for j in xrange(self.map_size[1]):
            self.tilelayer.extend([int(i) for i in f.readline().split()])
        #self.tilelayer_bg = []
        #for j in xrange(self.map_size[1]):
        #    self.tilelayer_bg.extend([int(i) for i in f.readline().split()])
        #self.tilelayer_fg = []
        #for j in xrange(self.map_size[1]):
        #    self.tilelayer_fg.extend([int(i) for i in f.readline().split()])
        self.occlayer = []
        for j in xrange(self.map_size[1]):
            self.occlayer.extend([int(i) for i in f.readline().split()])
        self.goaltiles = [int(i) for i in f.readline().split()]
        self.firetiles = [int(i) for i in f.readline().split()]
        self.powerpacktiles = [int(i) for i in f.readline().split()]
        numitems = [int(i) for i in f.readline().split()][0]
        self.vacsources = []
        for j in xrange(numitems):
            self.vacsources.append([int(i) for i in f.readline().split()])
        numitems = [int(i) for i in f.readline().split()][0]
        self.units = []
        for j in xrange(numitems):
            data = f.readline().split()
            self.units.append([int(data[0]),data[1],int(data[2])])
        self.switches = []
        num = [int(i) for i in f.readline().split()][0]
        for i in xrange(num):
            data = f.readline().split()
            self.switches.append([int(data[0]), int(data[1]), data[2], data[3], int(data[4])])
            for j in xrange(len(data)-5):
                self.switches[-1].append(int(data[j+5]))
        self.doors = []
        num = [int(i) for i in f.readline().split()][0]
        for i in xrange(num):
            data = f.readline().split()
            self.doors.append([int(data[0]), int(data[1]), int(data[2])])
        self.vacpumps = []
        num = [int(i) for i in f.readline().split()][0]
        for i in xrange(num):
            data = f.readline().split()
            self.vacpumps.append([int(data[0]), int(data[1]), int(data[2])])
        f.close()

class GameFont(object):
    def __init__(self,mainpath,path,size):
        self.fontim = pygame.image.load(os.path.join(mainpath,'data','gfx',path)).convert()
        self.fontim.set_colorkey((255,0,255))
        
        keys = list('0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ? = abcdefghijklmnopqrstuvwxyz+,-.!"#$%&`():')
        self.rects = {}
        for i in xrange(8):
            for j in xrange(10):
                k = keys[i*10+j]
                self.rects[k] = [j*size[0],i*size[1],size[0],size[1]]
        self.rects['i'][2] = size[1]/5
        self.rects['l'][2] = size[1]/5
    
    def RenderSentence(self, sentence, maxx, maxy, align='left'):
        sentsurf = pygame.Surface((maxx,maxy))
        sentsurf.fill((255,0,255))
        sentsurf.convert()
        sentsurf.set_colorkey((255,0,255))
        textheight = self.rects['A'][3]
        maxlines = maxy/textheight
        linedist = 0
        line = 0
        offsets = []
        if align in ['centre','vcentre']:
            for word in sentence.split(' '):
                letters = word.split()[0]+' '
                wordsize = sum([self.rects[let][2] for let in letters])
                if wordsize > maxx:
                    return False
                if (linedist+wordsize) >= maxx:
                    line += 1
                    offsets.append((maxx-linedist)/2)
                    linedist = 0
                    if line == maxlines:
                        return False
                for let in letters:
                    linedist += self.rects[let][2]
                if word[-1] == '\n':
                    line += 1
                    offsets.append((maxx-linedist)/2)
                    linedist = 0
            offsets.append((maxx-linedist)/2)
            voffset = maxy/2 - ((line+1)*textheight)/2
            linedist = 0
            line = 0
        for word in sentence.split(' '):
            letters = word.split()[0]+' '
            wordsize = sum([self.rects[let][2] for let in letters])
            if wordsize > maxx:
                return False
            if (linedist+wordsize) >= maxx:
                line += 1
                linedist = 0
                if line == maxlines:
                    return False
            for let in letters:
                if align == 'centre':
                    sentsurf.blit(self.fontim,(linedist+offsets[line],textheight*line),area=self.rects[let])
                elif align == 'vcentre':
                    sentsurf.blit(self.fontim,(linedist+offsets[line],textheight*line+voffset),area=self.rects[let])
                else:
                    sentsurf.blit(self.fontim,(linedist,textheight*line),area=self.rects[let])
                linedist += self.rects[let][2]
            if word[-1] == '\n':
                line += 1
                linedist = 0
        return sentsurf

def init(mainpath):
    
    # Load background and tile layer images
    global tiles
    global tiles_coords
    
    tiles = pygame.image.load(os.path.join(mainpath,'data','gfx','tilesheet_3x.png')).convert()
    tiles.set_colorkey((0,0,0))
    tiles_coords = []
    for j in xrange(36):
        for i in xrange(8):
            tiles_coords.append((i*27+3, j*27+3, 24, 24))
    
    # generate lookups for determining dark/light rendering according to vacuum (per tile! god this is annoying)
    global tilevaclookup
    tilevaclookup = []
    tilevaclookup.extend([8,8,8,8,0,0,0,0])
    tilevaclookup.extend([0,9,8,7,0,0,0,0])
    tilevaclookup.extend([0,6,5,4,0,0,0,0])
    tilevaclookup.extend([0,3,2,1,0,0,0,0])
    tilevaclookup.extend([0,1,3,2,0,0,0,0])
    tilevaclookup.extend([0,7,9,2,0,0,0,0])
    tilevaclookup.extend([0,3,2,1,0,0,0,0])
    tilevaclookup.extend([0,9,8,7,0,0,0,0])
    tilevaclookup.extend([8,2,4,6,0,0,0,0])
    tilevaclookup.extend([8,2,4,6,0,0,0,0])
    tilevaclookup.extend([8,2,4,6,0,0,0,0])
    tilevaclookup.extend([8,2,4,6,0,0,0,0])
    tilevaclookup.extend([8,2,4,6,0,0,0,0])
    tilevaclookup.extend([8,2,4,6,0,0,0,0])
    tilevaclookup.extend([8,2,4,6,0,0,0,0])
    tilevaclookup.extend([8,2,4,6,0,0,0,0])
    tilevaclookup.extend([8,2,4,6,0,0,0,0])
    tilevaclookup.extend([8,2,4,6,0,0,0,0])
    tilevaclookup.extend([8,2,4,6,0,0,0,0])
    tilevaclookup.extend([8,2,4,6,0,0,0,0])
    tilevaclookup.extend([8,2,4,6,0,0,0,0])
    tilevaclookup.extend([8,2,4,6,0,0,0,0])
    tilevaclookup.extend([0,0,0,0,0,0,0,0])
    tilevaclookup.extend([0,0,0,0,0,0,0,0])
    tilevaclookup.extend([0,0,0,0,0,0,0,0])
    tilevaclookup.extend([0,0,0,0,0,0,0,0])
    
    global singlestar
    singlestar = pygame.image.load(os.path.join(mainpath,'data','gfx','stars_3x.png')).convert()
    singlestar.set_colorkey((0,0,0))
    
    # switch sprites
    global switches_tiles
    switches_tiles = {}
    switches_tiles['normal'] = [80,64,88] # off, on, no power
    switches_tiles['once'] = [96,104] # empty power pack, full power pack
    
    # Door Sprites
    global door_tiles
    door_tiles = {}
    door_tiles['open'] = [3]
    door_tiles['closed'] = [1]
    door_tiles['frame'] = [0]
    
    # Vac Pump sprites
    global vacpump_tiles
    vacpump_tiles = [4,3] # pressurised, vacuum, no power
    
    # fire sprites
    global fire_tiles
    fire_tiles = [176,177,178,179]
    
    # powerpack tile
    global powerpack_tile
    powerpack_tile = 8
    
    # Load player sprites
    global player_spritesheet
    global player_spritesheet_coords
    global player_spriteseq
    
    player_spritesheet = {}
    player_spritesheet['right'] = pygame.image.load(os.path.join(mainpath,'data','gfx','units_3x.png')).convert()
    player_spritesheet['right'].set_colorkey((0,0,0))
    
    player_spritesheet['left'] = pygame.image.load(os.path.join(mainpath,'data','gfx','units_3x.png')).convert()
    player_spritesheet['left'].set_colorkey((0,0,0))
    
    player_spritesheet['floating'] = pygame.image.load(os.path.join(mainpath,'data','gfx','units_3x.png')).convert()
    player_spritesheet['floating'].set_colorkey((0,0,0))
    
    player_spritesheet_coords = []
    for j in xrange(5):
        for i in xrange(8):
            player_spritesheet_coords.append((i*30+3, j*33+3, 27, 27))
    
    tempsurf = pygame.image.load(os.path.join(mainpath,'data','gfx','units_3x.png')).convert()
    angles = [180,90,0,270]
    for j in [0,1,3]:
        for i in xrange(8):
            coords = (i*30+3, j*33+3, 27, 27)
            coords2 = (i*30+3, 2*33+3, 27, 27)
            tempsurf2 = pygame.Surface((27,27))
            tempsurf2.blit(tempsurf, (0,0), area=coords2)
            tempsurf3 = pygame.transform.rotate(tempsurf2,angles[j])
            player_spritesheet['floating'].blit(tempsurf3, (coords[0],coords[1]))
    for j in [0,1,3,4]:
        for i in xrange(8):
            coords = (i*30+3, j*33+3, 27, 27)
            tempsurf2 = pygame.Surface((27,27))
            tempsurf2.blit(tempsurf, (0,0), area=coords)
            tempsurf3 = pygame.transform.flip(tempsurf2,True,False)
            player_spritesheet['left'].blit(tempsurf3, (coords[0],coords[1]))
    #pygame.image.save(player_spritesheet['floating'],"test_float.png")
    #pygame.image.save(player_spritesheet['left'],"test_left.png")
    
    player_spriteseq = {}
    player_spriteseq['unit'] = {}
    player_spriteseq['astro'] = {}
    
    player_spriteseq['unit']['stand'] = [0,8]
    player_spriteseq['unit']['walk'] = [0]
    player_spriteseq['unit']['floating'] = [0,8,16,24]
    
    player_spriteseq['astro']['stand'] = [24,32]
    player_spriteseq['astro']['walk'] = [24]
    
    # Load Mouse control sprites
    global mouse_spritesheet
    global mouse_spritesheet_coords
    global mouse_spriteseq
    
    mouse_spritesheet = pygame.image.load(os.path.join(mainpath,'data','gfx','mousetemp_3x.png')).convert()
    mouse_spritesheet.set_colorkey((255,0,255))
    mouse_spritesheet_coords = {}
    mouse_spritesheet_coords['hover'] = []
    for i in xrange(4):
        mouse_spritesheet_coords['hover'].append((i*24, 0, 24, 24))
    mouse_spriteseq = {}
    mouse_spriteseq['hover'] = [0]
    
    # Load level data
    global initleveldata
    
    initleveldata = {}
    #initleveldata["level_default"] = InitLevelData(os.path.join(mainpath,"data","lvl","level_default.txt"))
    initleveldata["level_medium001"] = InitLevelData(os.path.join(mainpath,"data","lvl","level_medium001.txt"))
    initleveldata["level_solo"] = InitLevelData(os.path.join(mainpath,"data","lvl","level_solo.txt"))
    initleveldata["level_tutorial"] = InitLevelData(os.path.join(mainpath,"data","lvl","level_tutorial.txt"))
    initleveldata["level_twosections"] = InitLevelData(os.path.join(mainpath,"data","lvl","level_twosections.txt"))
    
    global levellist
    levellist = ["level_tutorial","level_medium001","level_solo","level_twosections"]
    
    # Sound Data
    global soundfx
    soundfx = {}
    soundfx['switch'] = pygame.mixer.Sound(os.path.join(mainpath,'data','snd','31589__freqman__buttons01.ogg'))
    soundfx['vacalarm'] = pygame.mixer.Sound(os.path.join(mainpath,'data','snd','vacuum_alarm.ogg'))
    soundfx['shipalarm'] = pygame.mixer.Sound(os.path.join(mainpath,'data','snd','shipalarm.ogg'))
    soundfx['burn'] = pygame.mixer.Sound(os.path.join(mainpath,'data','snd','steam_hisses_marker4.ogg'))
    soundfx['powerup'] = pygame.mixer.Sound(os.path.join(mainpath,'data','snd','powerup.ogg'))
    
    # Music Data
    global musicpaths
    musicpaths = {}
    musicpaths['mourningsong'] = os.path.join(mainpath,'data','music','mourningsong.ogg')
    musicpaths['vanes'] = os.path.join(mainpath,'data','music','vanes.ogg')
    musicpaths['intromusic002'] = os.path.join(mainpath,'data','music','intromusic002.ogg')
    musicpaths['herodown'] = os.path.join(mainpath,'data','music','herodown.ogg')
    musicpaths['explosionandalarm'] = os.path.join(mainpath,'data','music','explosionandalarm.ogg')
    
    # Fonts and text
    global gamefont_green
    global gamefont_red
    global gamefont_yellow
    gamefont_green = GameFont(mainpath,'broddmin_5x10_shad_green_2x.png',(10,20))
    gamefont_red = GameFont(mainpath,'broddmin_5x10_shad_red_2x.png',(10,20))
    gamefont_yellow = GameFont(mainpath,'broddmin_5x10_shad_yellow_2x.png',(10,20))
    
    global status_message
    status_message = {}
    
    status_message['background'] = pygame.Surface((592,24))
    status_message['background'].fill((0,0,0))
    status_message['background'].convert()
    status_message['background'].set_alpha(128)
    
    status_message['background2'] = pygame.Surface((592,48))
    status_message['background2'].fill((0,0,0))
    status_message['background2'].convert()
    status_message['background2'].set_alpha(128)
    
    status_message['unitnomove'] = gamefont_green.RenderSentence("I can`t find a way through, sir ..", 640, 48, align='vcentre')
    status_message['unitkilledfire'] = gamefont_red.RenderSentence("Crew killed ..", 640, 48, align='vcentre')
    status_message['unitkilledvacuum'] = gamefont_red.RenderSentence("Crew exposed to vacuum ..", 640, 48, align='vcentre')
    status_message['unitsaved'] = gamefont_green.RenderSentence("Crew member in safe zone ..", 640, 48, align='vcentre')
    status_message['unitnoselect'] = gamefont_green.RenderSentence("You need to select a unit first ..", 640, 48, align='vcentre')
    
    status_message['switchpower'] = gamefont_green.RenderSentence("This switch needs power first ..", 640, 48, align='vcentre')
    status_message['powerplugnochip'] = gamefont_green.RenderSentence("This power socket needs a power cell ..", 640, 48, align='vcentre')
    status_message['pickupcell'] = gamefont_green.RenderSentence("Picked up a power cell!", 640, 48, align='vcentre')
    status_message['powerplugactivate'] = gamefont_green.RenderSentence("Power on, switch activated!", 640, 48, align='vcentre')
    
    status_message['pumppartialsuccess'] = gamefont_green.RenderSentence("Not enough air to re-pressurise all rooms ..", 640, 48, align='vcentre')
    status_message['pumpdepressure'] = gamefont_green.RenderSentence("Compartment de-pressurised ..", 640, 48, align='vcentre')
    status_message['pumprepressure'] = gamefont_green.RenderSentence("Compartment re-pressurised ..", 640, 48, align='vcentre')
    status_message['pumpfailure'] = gamefont_green.RenderSentence("Re-pressurisation failed. Reduce the compartment volume by closing doors ..", 640, 48, align='vcentre')
    
    global assist_message
    assist_message = {}
    assist_message['crewunit'] = gamefont_green.RenderSentence("crew member\n (no suit)", 160, 48, align='vcentre')
    assist_message['crewastro'] = gamefont_green.RenderSentence("crew member\n (suit)", 160, 48, align='vcentre')
    assist_message['crewunitpowerpack'] = gamefont_green.RenderSentence("crew member\n (power cell)", 160, 48, align='vcentre')
    assist_message['crewastropowerpack'] = gamefont_green.RenderSentence("crew member\n (power cell)", 160, 48, align='vcentre')
    assist_message['crewsafe'] = gamefont_green.RenderSentence("crew member\n (escape pod)", 160, 48, align='vcentre')
    assist_message['crewdead'] = gamefont_red.RenderSentence("killed crew", 160, 48, align='vcentre')
    assist_message['goal'] = gamefont_green.RenderSentence("escape pod", 160, 48, align='vcentre')
    assist_message['door'] = gamefont_green.RenderSentence("door", 160, 48, align='vcentre')
    assist_message['airpump'] = gamefont_green.RenderSentence("air pump", 160, 48, align='vcentre')
    assist_message['switch'] = gamefont_green.RenderSentence("switch", 160, 48, align='vcentre')
    assist_message['switchnopower'] = gamefont_green.RenderSentence("switch\n (needs power)", 160, 48, align='vcentre')
    assist_message['powerplug'] = gamefont_green.RenderSentence("power socket", 160, 48, align='vcentre')
    assist_message['powerplugempty'] = gamefont_green.RenderSentence("power socket\n (needs cell)", 160, 48, align='vcentre')
    assist_message['powerchip'] = gamefont_green.RenderSentence("power cell", 160, 48, align='vcentre')
    assist_message['powerchipinsert'] = gamefont_green.RenderSentence("insert power cell", 160, 48, align='vcentre')
    assist_message['fire'] = gamefont_yellow.RenderSentence("fire", 160, 48, align='vcentre')
    
    assist_message['depressurise'] = gamefont_green.RenderSentence("depressurise", 160, 48, align='vcentre')
    assist_message['repressurise'] = gamefont_green.RenderSentence("repressurise", 160, 48, align='vcentre')
    assist_message['opendoor'] = gamefont_green.RenderSentence("open door", 160, 48, align='vcentre')
    assist_message['closedoor'] = gamefont_green.RenderSentence("close door", 160, 48, align='vcentre')
    
    global hud_data
    hud_data = {}
    
    hud_data['launch'] = gamefont_green.RenderSentence("launch escape pod", 180, 48, align='vcentre')
    hud_data['failure'] = gamefont_red.RenderSentence("mission failed", 180, 48, align='vcentre')
    hud_data['launchcolours'] = [(58,58,58),(0,117,35),(0,179,53),(255,255,255)]
    hud_data['launchpos'] = ((390,428),(190,30))
    
    hud_data['panel_surf'] = pygame.image.load(os.path.join(mainpath,'data','gfx','ui_3x.png')).convert()
    hud_data['panel_surf'].set_colorkey((0,0,0))
    hud_data['panel_surf'].convert()
    
    hud_data['panel_bg'] = pygame.Surface((200*3,20*3))
    hud_data['panel_bg'].set_colorkey((0,0,0))
    hud_data['panel_bg'].blit(hud_data['panel_surf'], (0,0), area=(3,3,200*3,20*3))
    hud_data['panel_bg'].convert()
    hud_data['panel_bg'].set_alpha(128)
    
    hud_data['panel_launch'] = pygame.Surface((29*3,10*3))
    hud_data['panel_launch'].set_colorkey((0,0,0))
    hud_data['panel_launch'].convert()
    hud_data['panel_launch'].blit(hud_data['panel_surf'], (0,0), area=(171*3,22*3,29*3,10*3))
    hud_data['panel_launch_coords'] = (171*3+21,10*3+414)
    
    hud_data['panel_abandon'] = pygame.Surface((36*3,10*3))
    hud_data['panel_abandon'].set_colorkey((0,0,0))
    hud_data['panel_abandon'].convert()
    hud_data['panel_abandon'].blit(hud_data['panel_surf'], (0,0), area=(133*3,22*3,36*3,10*3))
    hud_data['panel_abandon_coords'] = (133*3+21,10*3+414)
    
    hud_data['panel_statusbox'] = {}
    hud_data['panel_statusbox']['alive'] = pygame.Surface((10*3,10*3))
    hud_data['panel_statusbox']['alive'].set_colorkey((0,0,0))
    hud_data['panel_statusbox']['alive'].convert()
    hud_data['panel_statusbox']['alive'].blit(hud_data['panel_surf'], (0,0), area=(2*3,22*3,10*3,10*3))
    hud_data['panel_statusbox']['safe'] = pygame.Surface((10*3,10*3))
    hud_data['panel_statusbox']['safe'].set_colorkey((0,0,0))
    hud_data['panel_statusbox']['safe'].convert()
    hud_data['panel_statusbox']['safe'].blit(hud_data['panel_surf'], (0,0), area=(13*3,22*3,10*3,10*3))
    hud_data['panel_statusbox']['dead'] = pygame.Surface((10*3,10*3))
    hud_data['panel_statusbox']['dead'].set_colorkey((0,0,0))
    hud_data['panel_statusbox']['dead'].convert()
    hud_data['panel_statusbox']['dead'].blit(hud_data['panel_surf'], (0,0), area=(24*3,22*3,10*3,10*3))
    
    hud_data['panel_statusbox_coords'] = (2*3,10*3)
    
    hud_data['panel_facesunits'] = []
    for i in xrange(8):
        hud_data['panel_facesunits'].append(pygame.Surface((8*3,8*3)))
        hud_data['panel_facesunits'][-1].set_colorkey((0,0,0))
        hud_data['panel_facesunits'][-1].convert()
        hud_data['panel_facesunits'][-1].blit(hud_data['panel_surf'], (0,0), area=(126*3+i*8*3,54*3,8*3,8*3))
    hud_data['panel_facesastro'] = []
    for i in xrange(4):
        hud_data['panel_facesastro'].append(pygame.Surface((8*3,8*3)))
        hud_data['panel_facesastro'][-1].set_colorkey((0,0,0))
        hud_data['panel_facesastro'][-1].convert()
        hud_data['panel_facesastro'][-1].blit(hud_data['panel_surf'], (0,0), area=(126*3+i*8*3,62*3,8*3,8*3))
    
    # Mentor data
    global mentor_data, mentor_messages
    
    mentor_data = {}
    mentor_data['panel_facementor'] = pygame.Surface((16*3,16*3))
    mentor_data['panel_facementor'].convert()
    mentor_data['panel_facementor'].blit(hud_data['panel_surf'], (0,0), area=(3*3,35*3,16*3,16*3))
    
    mentor_messages = {}
    
    mentor_messages['startmsg'] = {}
    mentor_messages['startmsg']['level_tutorial'] = ["Crewman! come in, I repeat, come in ...","Thank goodness, you`re alive!","There`s been a major catastrophe on board the Endeavour.","The ship has broken up into multiple sections, many dead and injured.","I`m calling in from an re-group site where myself and other members of the ship`s science team are currently sheltered.","Luckily for us, our section of the ship wasn`t so badly damaged.","We`ve managed to track the section of wreckage you are currently stranded on over a partially-working radar.","I need you to try and get to an escape pod and rendezvous with us.","You look a little dazed, I think you must have blacked out during the explosion.","Don`t worry, I`ll guide you to the pod over the communicator.","First you need to get out of your cabin, there`s a switch on the wall in front of you that opens the door.","Use the mouse to click on yourself and then click the switch to walk over and activate it."]
    
    mentor_messages['startmsg']['level_medium001'] = ["You`re currently patched in to the survivors.","Notice the unit to the bottom of your screen.","He is wearing a space suit and can safely traverse areas flooded with vacuum.","Get the survivors to the escape pod!"]
    
    mentor_messages['startmsg']['level_twosections'] = ["Sometimes hard decisions have to be made, my friend."]
    
    mentor_messages['startmsgsurf'] = {}
    for k in mentor_messages['startmsg'].keys():
        mentor_messages['startmsgsurf'][k] = []
        for m in mentor_messages['startmsg'][k]:
            mentor_messages['startmsgsurf'][k].append(gamefont_green.RenderSentence(m, 590, 48, align='vcentre'))
    
    mentor_messages['tiles'] = [303, 375, 387, 498, 611, 979]
    mentor_messages['msg'] = [["Watch out! there`s fire down that corridor."],\
        ["Be careful when opening airlocks!","Dark areas of the ship have been exposed to space ...",'you`re not currently wearing a spacesuit!'],\
        ["Be careful when opening airlocks!","Dark areas of the ship have been exposed to space ...",'you`re not currently wearing a spacesuit!'],\
        ["There`s a switch below you that operates a door further along.","Notice that the switch light is grey.","This means the switch is currently unpowered.","There should be a power socket nearby","If you can find a power cell lying around, plug it in to get the switch working again."],\
        ["There`s a room ahead with three switches for three doors in the adjacent corridor.","One of those doors is an airlock, exposed to space.","See if you can`t open the airlock and extinguish the fire by exposing it to vacuum.","Be careful to close the surrounding doors first!","You don`t want to expose yourself to vacuum at the same time!","Once the fire is extinguished, you can close the airlock again.",'The air pump system in the room should bring the air pressure back.',"Any sealed off areas in the ship should be filled with air and safe to traverse."],\
        ["Up ahead there`s an escape pod.",'When you get inside, click the launch button.','The pod is programmed to head to a pre-arranged evacuation site where we are currently located.','Once you are here, I`ll get you connected up over comms to other stranded groups.','I need you to guide as many survivors as possible back here so we can re-group and contact Earth.','In some sections of the wreckage there are multiple crew and even some crew with undamaged spacesuits.','You need to take command at each site and get the surviving crew to work together to escape.','Even if you can get atleast one survivor into an escape pod, it`s internal systems will be able to guide it back here.','Good luck, crewman, and god speed!']]
    
    mentor_messages['msgsurf'] = []
    for mg in mentor_messages['msg']:
        mentor_messages['msgsurf'].append([])
        for m in mg:
            mentor_messages['msgsurf'][-1].append(gamefont_green.RenderSentence(m, 590, 48, align='vcentre'))
    
    #print assist_message.values()
    
    # Title screen data
    global titlescreen_data
    
    titlescreen_data = {}
    
    titlescreen_data['titletext'] = pygame.image.load(os.path.join(mainpath,'data','gfx','title.png')).convert()
    titlescreen_data['titletext'].set_colorkey((0,0,0))
    
    titlescreen_data['newgame_off'] = gamefont_green.RenderSentence("New Game", 100, 24, align='vcentre')
    titlescreen_data['newgame_on'] = gamefont_red.RenderSentence("New Game", 100, 24, align='vcentre')
    titlescreen_data['continue_off'] = gamefont_green.RenderSentence("Continue", 100, 24, align='vcentre')
    titlescreen_data['continue_on'] = gamefont_red.RenderSentence("Continue", 100, 24, align='vcentre')
    titlescreen_data['newgamepos'] = (270,300)
    titlescreen_data['continuepos'] = (270,340)
    
    # Menu Data
    global pausemenu_data
    
    pausemenu_data = {}
    pausemenu_data['abandon'] = gamefont_green.RenderSentence("abandom crew?", 160, 24, align='vcentre')
    pausemenu_data['yes_off'] = gamefont_green.RenderSentence("yes", 60, 24, align='vcentre')
    pausemenu_data['yes_on'] = gamefont_red.RenderSentence("yes", 60, 24, align='vcentre')
    pausemenu_data['no_off'] = gamefont_green.RenderSentence("no", 60, 24, align='vcentre')
    pausemenu_data['no_on'] = gamefont_red.RenderSentence("no", 60, 24, align='vcentre')
    pausemenu_data['reset_off'] = gamefont_green.RenderSentence("reset level", 160, 24, align='vcentre')
    pausemenu_data['reset_on'] = gamefont_red.RenderSentence("reset level", 160, 24, align='vcentre')
    
    pausemenu_data['abandonpos'] = (250,260)
    pausemenu_data['yespos'] = (220,290)
    pausemenu_data['nopos'] = (300,290)
    pausemenu_data['resetpos'] = (340,290)
    
    # cutscene data
    global cutscenecurrentmusic
    cutscenecurrentmusic = 'none'
    
    global quotescene_data
    quotescene_data = [[ ['quote',[]] ],'none','titlescreen',[]]
    
    global logocutscene_data
    logocutscene_data = [[ ['team_chimera',[]] ],'none','quotescene',quotescene_data]
    
    global chap1_1_data, chap1_2_data, chap1_3_data
    chap1_3_data = [[ ['chapter1explode',[]] ],'explosionandalarm','maingame',[True, levellist[0]]]
    chap1_2_data = [[ ['chapter1text1',[]] ],'none','chapter1scene3',chap1_3_data]
    chap1_1_data = [[ ['chapter1',[]] ],'none','chapter1scene2',chap1_2_data]
    
    global prologue1_data, prologue2_data, prologue3_data, prologue4_data
    prologue4_data = [[ ['introtext3',[]] ],'herodown','chapter1scene1',chap1_1_data]
    prologue3_data = [[ ['shipsearth',[]] ],'herodown','prologuescene4',prologue4_data]
    prologue2_data = [[ ['introtext1',[]] ],'herodown','prologuescene3',prologue3_data]
    prologue1_data = [[ ['prologue',[]] ],'none','prologuescene2',prologue2_data]
    
    global chap2_1_data, chap2_2_data, chap2_3_data, chap2_4_data, chap2_5_data, chap2_6_data, chap2_7_data, chap2_8_data
    
    chap2_8_data = [[ ['chapter2actuald',[]] ],'none','maingame',[True, levellist[1]]]
    chap2_7_data = [[ ['chapter2actualc',[]] ],'none','chapter2scene7',chap2_8_data]
    chap2_6_data = [[ ['chapter2actualb',[]] ],'none','chapter2scene6',chap2_7_data]
    chap2_5_data = [[ ['chapter2actuala',[]] ],'none','chapter2scene5',chap2_6_data]
    chap2_4_data = [[ ['chapter2actual',[]] ],'none','chapter2scene5',chap2_5_data]
    chap2_3_data = [[ ['chapter2c',[]] ],'none','chapter2scene4',chap2_4_data]
    chap2_2_data = [[ ['chapter2b',[]] ],'none','chapter2scene3',chap2_3_data]
    chap2_1_data = [[ ['chapter2',[]] ],'none','chapter2scene2',chap2_2_data]
    
    global chap3_1_data, chap3_2_data, chap3_3_data, chap3_4_data
    chap3_4_data = [[ ['chapter3c',[]] ],'none','maingame',[True, levellist[2]]]
    chap3_3_data = [[ ['chapter3b',[]] ],'none','chapter3scene4',chap3_4_data]
    chap3_2_data = [[ ['chapter3a',[]] ],'none','chapter3scene3',chap3_3_data]
    chap3_1_data = [[ ['chapter3',[]] ],'none','chapter3scene2',chap3_2_data]
    
    global chap4_1_data, chap4_2_data, chap4_3_data
    chap4_3_data = [[ ['chapter4b',[]] ],'none','maingame',[True, levellist[3]]]
    chap4_2_data = [[ ['chapter4a',[]] ],'none','chapter4scene3',chap4_3_data]
    chap4_1_data = [[ ['chapter4',[]] ],'none','chapter4scene2',chap4_2_data]
    
    global chap5_1_data, chap5_2_data, chap5_3_data
    chap5_3_data = [[ ['epilogueb',[]] ],'none','titlescreen',[]]
    chap5_2_data = [[ ['epiloguea',[]] ],'none','chapter5scene3',chap5_3_data]
    chap5_1_data = [[ ['epilogue',[]] ],'none','chapter5scene2',chap5_2_data]
    
    # cutscene surface data
    global cutscenesurfs
    cutscenesurfs = {}
    
    cutscenesurfs['team_chimera'] = pygame.image.load(os.path.join(mainpath,'data','gfx','team_chimera_5x.png')).convert()
    cutscenesurfs['team_chimera'].set_colorkey((255,0,255))
    
    cutscenesurfs['quote'] = gamefont_yellow.RenderSentence('"... those brave souls that venture into the void of space face perils from both the outside and from within ..."', 600, 480, align='vcentre')
    cutscenesurfs['quote'].set_colorkey((255,0,255))
    
    cutscenesurfs['prologue'] = gamefont_yellow.RenderSentence('Prologue', 600, 480, align='vcentre')
    cutscenesurfs['introtext1'] = gamefont_yellow.RenderSentence('In the twentieth century we put a man on the moon. In the twenty second century, we reached out for the stars.', 600, 480, align='vcentre')
    introtext2 = gamefont_yellow.RenderSentence('The enormous colony ships left earth, venturing out into the void, destined for nearby stars and newly discovered inhabitable planets. The journeys were decades long, travelling at sub-light speeds, the crew to live out their lives at their destination, never to step foot on earth again.', 600, 480, align='vcentre')
    cutscenesurfs['shipsearth'] = pygame.image.load(os.path.join(mainpath,'data','gfx','shipsearth.png')).convert()
    cutscenesurfs['shipsearth'].blit(introtext2, (20,140))
    cutscenesurfs['introtext3'] = gamefont_yellow.RenderSentence('Far from the safety of our home planet, and beyond assistance, ships were occasionally never heard from again.', 600, 200, align='vcentre')

    cutscenesurfs['chapter1'] = gamefont_green.RenderSentence('Chapter 1', 600, 480, align='vcentre')
    cutscenesurfs['chapter1text1'] = gamefont_green.RenderSentence('Starship Endeavour, 276 crew, on route to Epsilon Eridani.\n May 7 2162: two years into the journey from Earth.', 600, 480, align='vcentre')
    cutscenesurfs['chapter1explode'] = pygame.Surface((640,480))
    cutscenesurfs['chapter1explode'].fill((0,0,0))
    starfield = MovingStarField()
    starfield.Draw(cutscenesurfs['chapter1explode'])
    
    cutscenesurfs['chapter2actual'] = gamefont_red.RenderSentence('Chapter 2', 600, 480, align='vcentre')
    cutscenesurfs['chapter2'] = gamefont_green.RenderSentence('You`re here at last! We`ve currently picked up about 45 survivors. I guess I should get you up to speed with what we know about the disaster, which unfortunately is very little.', 600, 480, align='vcentre')
    cutscenesurfs['chapter2b'] = gamefont_green.RenderSentence('There was an explosion in the engine room just before 9:00 hours. It`s unclear how this could have occurred .. there`s not many survivors from the engineering team to help fill us in on possible causes.', 600, 480, align='vcentre')
    cutscenesurfs['chapter2c'] = gamefont_green.RenderSentence('Get to the control room. I`m going to patch you in to another group of survivors we have just made contact with in a section of floating wreckage. I need to you take command and help get them to a nearby pod.', 600, 480, align='vcentre')
    cutscenesurfs['chapter2'].blit(mentor_data['panel_facementor'], (290,140))
    cutscenesurfs['chapter2b'].blit(mentor_data['panel_facementor'], (296,140))
    cutscenesurfs['chapter2c'].blit(mentor_data['panel_facementor'], (296,140))
    
    cutscenesurfs['chapter2actuala'] = gamefont_red.RenderSentence('From the diary of crewman Edmonds, J:\n Engine Technician, 3rd class\n Jan 25, 2160', 600, 480, align='vcentre')
    cutscenesurfs['chapter2actualb'] = gamefont_red.RenderSentence('Today we left Earth behind. I don`t know what to feel. Earth .. all its politics and overpopulation. I never thought I`d miss that. But now it`s sunk in. We`re never going back. Not ever.', 600, 480, align='vcentre')
    cutscenesurfs['chapter2actualc'] = gamefont_red.RenderSentence('A one way trip to Epsilon Eridani we use fifteen billion tonnes of fuel to get there .. that`s point 996 of the total mass of the Endeavour. You try to make it all the way there and have enough fueld to make the trip back to Earth, you need a ship ten times the size. The corporation would never fund that, colonist`s sanity ain`t worth it. God damn corporation.', 600, 480, align='vcentre')
    cutscenesurfs['chapter2actuald'] = gamefont_red.RenderSentence('Sure, I got plenty of work to keep me busy, keep my mind off things, but the fact remains. The rest of my life is going to be amongst the stars.', 600, 480, align='vcentre')
    
    cutscenesurfs['chapter3'] = gamefont_red.RenderSentence('Chapter 3', 600, 480, align='vcentre')
    cutscenesurfs['chapter3a'] = gamefont_red.RenderSentence('From the diary of crewman Edmonds, J:\n Engine Technician, 3rd class\n Sep 24, 2160', 600, 480, align='vcentre')
    cutscenesurfs['chapter3b'] = gamefont_red.RenderSentence('That god damn chief engineer O`Halloran. Don`t tell me how to do my god damn job. When we get off this stinking ship, He`ll be the first to go.', 600, 480, align='vcentre')
    cutscenesurfs['chapter3c'] = gamefont_red.RenderSentence('I can`t take this anymore, I`m surrounded by fools. They don`t get it. We`re nothing now, nothing. Just a drop in the ocean, a speck of dust in all the vastness of space. They think we`re all part of some grand plan, to better humanity, take up our place in the stars. But that`s just it: maybe our place ain`t in the stars. Maybe we shouldn`t be here.\n Sometimes I can feel the void closing in on me. It`s too much.', 600, 480, align='vcentre')
    
    cutscenesurfs['chapter4'] = gamefont_red.RenderSentence('Chapter 4', 600, 480, align='vcentre')
    cutscenesurfs['chapter4a'] = gamefont_red.RenderSentence('From the diary of crewman Edmonds, J:\n Engine Technician, 3rd class\n Nov 24, 2161', 600, 480, align='vcentre')
    cutscenesurfs['chapter4b'] = gamefont_red.RenderSentence('You know, it would be so easy to do it. Tweak a few settings here, a few settings there. Engines start to overheat. Get the mix of injector chemicals into the reactor control lines at just the right time and, BOOM! I bet O`Halloran thinks I`m too stupid for that sort of thing, but if there`s one thing I ain`t, I ain`t stupid. I bet he`d get a real shock if knew how easy it would be for me.', 600, 480, align='vcentre')
    
    cutscenesurfs['epilogue'] = gamefont_red.RenderSentence('Epilogue', 600, 480, align='vcentre')
    cutscenesurfs['epiloguea'] = gamefont_red.RenderSentence('May 20, 2162', 600, 480, align='vcentre')
    cutscenesurfs['epilogueb'] = gamefont_red.RenderSentence('Well, this is it. Computer read out says I`ve got 16 hours of air left. Stuck on this stinking piece of wreckage, trapped in a room with switches on the outside, no suit on the inside. God damn it. But that`s just it .. this is how it`s been for the last two years. Trapped in a tin can, kinetic energy too high to ever get back. I was dead when we left.\n Yeah, it was me alright. I did it. I finally did it after all these months. God damn I miss Earth.', 600, 480, align='vcentre')
    
    #print pausemenu_data.values()

class MovingStarField(object):
    def __init__(self):
        self.starposx1 = []
        self.starposy1 = []
        self.starsize1 = []
        self.starposx2 = []
        self.starposy2 = []
        self.starsize2 = []
        self.starN = 25
        for i in xrange(self.starN):
            self.starposx1.append(random.randint(-5,634))
            self.starposy1.append(random.randint(-5,474))
            val = random.randint(0,3)
            if val == 0:
                self.starsize1.append('big')
            else:
                self.starsize1.append('small')
        for i in xrange(self.starN):
            self.starposx2.append(random.randint(-5,634))
            self.starposy2.append(random.randint(-5,474))
            val = random.randint(0,3)
            if val == 0:
                self.starsize2.append('big')
            else:
                self.starsize2.append('small')
    
    def Update(self):
        for i in xrange(self.starN):
            self.starposx1[i] += 2
            if self.starposx1[i] > 640:
                self.starposx1[i] = 0
                self.starposy1[i] = random.randint(-5,474)
            self.starposx2[i] += 4
            if self.starposx2[i] > 640:
                self.starposx2[i] = 0
                self.starposy2[i] = random.randint(-5,474)
    
    def Draw(self, screen):
        for i in xrange(self.starN):
            if self.starsize1[i] == 'big':
                screen.blit(singlestar, (self.starposx1[i],self.starposy1[i]), area=(0,0,9,9))
            else:
                screen.blit(singlestar, (self.starposx1[i],self.starposy1[i]), area=(9,0,9,9))
        for i in xrange(self.starN):
            if self.starsize2[i] == 'big':
                screen.blit(singlestar, (self.starposx2[i],self.starposy2[i]), area=(0,0,9,9))
            else:
                screen.blit(singlestar, (self.starposx2[i],self.starposy2[i]), area=(9,0,9,9))