#!/usr/bin/python

"""
tilemap.py - tilemap classes
"""

import os
import pygame
from pygame.locals import *
from gamedirector import *
from math import *

import resources
import units
import path

class Switch(object):
    def __init__(self,id,tile1,tile2,type,mode,powerstate,h_assoc):
        self.id = id
        self.state = 0
        self.tile1 = tile1
        self.tile2 = tile2
        self.type = type
        self.mode = mode
        self.powerstate = powerstate
        self.h_assoc = h_assoc
    
    def Trigger(self):
        if self.state >= 0 and self.powerstate == 1:
            activated = False
            for h in self.h_assoc:
                activated = True
                if self.type == 'powerplug':
                    resources.soundfx['powerup'].play()
                    h.powerstate = 1
                else:
                    resources.soundfx['switch'].play()
                    if h.state == True:
                        h.state = False
                    else:
                        h.state = True
                    h.Update()
            if activated:
                self.state = (self.state+1) % 2
            if self.mode == 'once':
                self.state = -1
                
    def TriggerOff(self):
        if self.mode == 'mom':
            activated = False
            for h in self.h_assoc:
                activated = True
                if h.state == True:
                    h.state = False
                else:
                    h.state = True
            if activated:
                resources.soundfx['switch'].play()

class SwitchControl(object):
    def __init__(self,parent):
        self.parent = parent
        self.switches = []
    
    def AddSwitch(self,tile1,tile2,type,mode,powerstate,h_assoc):
        id = len(self.switches)+400
        self.switches.append(Switch(id,tile1,tile2,type,mode,powerstate,h_assoc))
        self.parent.InsertObj(tile1,id)
        msx = self.parent.map_size[0]
        if tile2-tile1 == msx:
            self.switches[-1].orient = 0
        elif tile1-tile2 == msx:
            self.switches[-1].orient = 1
        elif tile1-tile2 == 1:
            self.switches[-1].orient = 2
        else:
            self.switches[-1].orient = 3
    
    def DrawSwitches(self,screen):
        (camerax,cameray) = (self.parent.h_camera.x-self.parent.h_camera.w_view/2,self.parent.h_camera.y-self.parent.h_camera.h_view/2)
        for s in self.switches:
            coords = [(s.tile1%self.parent.map_size[0])*self.parent.tilesize, (s.tile1/self.parent.map_size[0])*self.parent.tilesize]
            if s.type == 'powerplug':
                if s.state >= 0:
                    tileval = resources.switches_tiles[s.mode][0]+s.orient
                else:
                    tileval = resources.switches_tiles[s.mode][1]+s.orient
            else:
                if s.powerstate == 0:
                    tileval = resources.switches_tiles[s.mode][2]+s.orient
                elif s.h_assoc[0].state == 0:
                    tileval = resources.switches_tiles[s.mode][0]+s.orient
                else:
                    tileval = resources.switches_tiles[s.mode][1]+s.orient
                
            if self.parent.airlayer[s.tile2] == 0:
                tileval += 4
            tilecoords = resources.tiles_coords[tileval]
            screen.blit(resources.tiles, (coords[0]-camerax,coords[1]-cameray), area=tilecoords)

class Door(object):
    def __init__(self,id,tile1,tile2,state,h_tiledlayers):
        self.id = id
        self.tile1 = tile1
        self.tile2 = tile2
        self.state = state
        self.h_tiledlayers = h_tiledlayers
        
        #self.tilelist = [tile1,tile2]
        self.tilelist = []
        mint = min(tile1,tile2)
        maxt = max(tile1,tile2)
        mapsizex = self.h_tiledlayers.map_size[0]
        if (mint/mapsizex) == (maxt/mapsizex): # on same row, horizontal door
            self.direction = 'horizontal'
            for i in xrange(mint+1,maxt):
                self.tilelist.append(i)
        else:
            self.direction = 'vertical'
            for i in xrange(mint+mapsizex,maxt,mapsizex):
                self.tilelist.append(i)
            
        #self.nframes = 4
        #self.ticksperframe = 3
        #self.closedpos = self.nframes*self.ticksperframe
        #if self.opentype == 0:
        #    self.position = self.closedpos # fully closed
        #elif self.opentype == 1:
        #    self.position = 0 # fully open
    
    def Update(self):
        mapsizex = self.h_tiledlayers.map_size[0]
        occlayer = self.h_tiledlayers.occlayer
        if self.state == 1:
            occval = 0
        elif self.state == 0:
            occval = 1
        for t in self.tilelist:
            occlayer[t] = occval

class DoorControl(object):
    def __init__(self,parent):
        self.parent = parent
        self.doors = []
        
    def AddDoor(self,tile1,tile2,state):
        id = len(self.doors)+500
        self.doors.append(Door(id,tile1,tile2,state,self.parent))
        self.doors[-1].Update()
        for t in self.doors[-1].tilelist:
            self.parent.InsertObj(t,id)
    
    def UpdateDoors(self):
        for d in self.doors:
            pass
            """
            if self.opentype == 'closed' and not self.state: # closed door, switched off
                self.position = min(self.position+1,self.closedpos)
            elif self.opentype == 'closed' and self.state: # closed door, switched on
                self.position = max(self.position-1,0)
            elif self.opentype == 'open' and not self.state: # open door, switched off
                self.position = max(self.position-1,0)
            elif self.opentype == 'open' and self.state: # open door, switched on
                self.position = min(self.position+1,self.closedpos)
            if self.position == 0:
                self.SetDoorOcc('open')
            else:
                self.SetDoorOcc('closed')
            """
    
    def DrawDoors(self,screen):
        (camerax,cameray) = (self.parent.h_camera.x-self.parent.h_camera.w_view/2,self.parent.h_camera.y-self.parent.h_camera.h_view/2)
        ts = self.parent.tilesize
        msx = self.parent.map_size[0]
        for d in self.doors:
            for t in d.tilelist:
                if d.state == 0: # closed
                    tileval = resources.door_tiles['closed'][0]
                else:
                    tileval = resources.door_tiles['open'][0]
                if self.parent.airlayer[t+msx] == 0:
                    tileval += 4
                tilecoords = resources.tiles_coords[tileval]
                x1 = ts*(t%msx)
                y1 = ts*(t/msx)
                screen.blit(resources.tiles, (x1-camerax,y1-cameray), area=tilecoords)
    
    def DrawDoorFrames(self,screen):
        (camerax,cameray) = (self.parent.h_camera.x-self.parent.h_camera.w_view/2,self.parent.h_camera.y-self.parent.h_camera.h_view/2)
        ts = self.parent.tilesize
        msx = self.parent.map_size[0]
        for d in self.doors:
            for t in d.tilelist:
                tileval = resources.door_tiles['frame'][0]
                if self.parent.airlayer[t+msx] == 0:
                    tileval += 4
                tilecoords = resources.tiles_coords[tileval]
                x1 = ts*(t%msx)
                y1 = ts*(t/msx)
                screen.blit(resources.tiles, (x1-camerax,y1-cameray), area=tilecoords)

class VacPump(object):
    def __init__(self,id,tile1,tile2,maxvolume,h_tiledlayers):
        self.id = id
        self.tile1 = tile1
        self.tile2 = tile2
        self.maxvolume = maxvolume
        self.h_tiledlayers = h_tiledlayers
        self.state = False
    
    def Update(self):
        if self.h_tiledlayers.airlayer[self.tile2] == 1:
            self.state = True
        else:
            self.state = False

class VacPumpControl(object):
    def __init__(self,parent):
        self.parent = parent
        self.vacpumps = []
        
    def AddVacPump(self,tile1,tile2,maxvolume):
        id = len(self.vacpumps)+600
        self.vacpumps.append(VacPump(id,tile1,tile2,maxvolume,self.parent))
        self.vacpumps[-1].Update()
        self.parent.InsertObj(tile1,id)
    
    def DrawVacPumps(self,screen):
        (camerax,cameray) = (self.parent.h_camera.x-self.parent.h_camera.w_view/2,self.parent.h_camera.y-self.parent.h_camera.h_view/2)
        ts = self.parent.tilesize
        msx = self.parent.map_size[0]
        for p in self.vacpumps:
            if p.state == True:
                tilecoords = resources.tiles_coords[resources.vacpump_tiles[0]]
            elif p.state == False:
                tilecoords = resources.tiles_coords[resources.vacpump_tiles[1]]
            x1 = ts*(p.tile1%msx)
            y1 = ts*(p.tile1/msx)
            screen.blit(resources.tiles, (x1-camerax,y1-cameray), area=tilecoords)

class FireControl(object):
    def __init__(self,firetiles,parent):
        self.parent = parent
        self.firetiles = firetiles[:]
        for i in xrange(len(self.firetiles)):
            self.parent.InsertObj(self.firetiles[i],i+200)
        self.nframes = 4
        self.ticks = 0
        self.nticksperframe = 3
        self.frame = 0
    
    def UpdateFires(self):
        for i in xrange(len(self.firetiles)):
            ft = self.firetiles[i]
            if ft == -1:
                continue
            if self.parent.airlayer[ft] == 0:
                self.firetiles[i] = -1
                self.parent.RemoveObj(ft,i+200)
        if (self.ticks%self.nticksperframe) == 0:
            self.frame = (self.frame+1)%self.nframes
        self.ticks += 1
    
    def DrawFires(self,screen):
        (camerax,cameray) = (self.parent.h_camera.x-self.parent.h_camera.w_view/2,self.parent.h_camera.y-self.parent.h_camera.h_view/2)
        ts = self.parent.tilesize
        msx = self.parent.map_size[0]
        for i in xrange(len(self.firetiles)):
            ft = self.firetiles[i]
            if ft == -1:
                continue
            frame = (self.frame+i)%self.nframes
            tilecoords = resources.tiles_coords[resources.fire_tiles[frame]]
            x1 = ts*(ft%msx)
            y1 = ts*(ft/msx)
            screen.blit(resources.tiles, (x1-camerax,y1-cameray), area=tilecoords)

class PowerPackControl(object):
    def __init__(self,powerpacktiles,parent):
        self.parent = parent
        self.powerpacktiles = powerpacktiles[:]
        for i in xrange(len(self.powerpacktiles)):
            self.parent.InsertObj(self.powerpacktiles[i],i+700)
    
    def DrawPowerPacks(self,screen):
        (camerax,cameray) = (self.parent.h_camera.x-self.parent.h_camera.w_view/2,self.parent.h_camera.y-self.parent.h_camera.h_view/2)
        ts = self.parent.tilesize
        msx = self.parent.map_size[0]
        for i in xrange(len(self.powerpacktiles)):
            ppt = self.powerpacktiles[i]
            if ppt == -1:
                continue
            #frame = (self.frame+i)%self.nframes
            #tilecoords = resources.tiles_coords[resources.powerpack_tiles[frame]]
            tilecoords = resources.tiles_coords[resources.powerpack_tile]
            x1 = ts*(ppt%msx)
            y1 = ts*(ppt/msx)
            screen.blit(resources.tiles, (x1-camerax,y1-cameray), area=tilecoords)

class EscapePod(object):
    def __init__(self,parent,tile):
        self.parent = parent
        self.launch = False
        self.tile = tile
        self.speed = 2
        self.x = self.parent.tilesize*(tile%self.parent.map_size[0])
        self.relx = 0
        self.y = self.parent.tilesize*(tile/self.parent.map_size[0] - 4)
        self.surf = pygame.Surface((24*7,24*9))
        self.surf.fill((0,0,0))
        self.surf.convert()
        self.surf.set_colorkey((0,0,0))
        self.surf.blit(resources.tiles, (0,0), area=(3,732,24*7,24*9))
    
    def Update(self):
        if self.launch == True:
            self.x += self.speed
            self.relx += self.speed
    
    def Draw(self,screen):
        (camerax,cameray) = (self.parent.h_camera.x-self.parent.h_camera.w_view/2,self.parent.h_camera.y-self.parent.h_camera.h_view/2)
        screen.blit(self.surf, (self.x-camerax,self.y-cameray))

class TiledLayers(object):
    
    def __init__(self,parent):
        self.parent = parent
    
    def set_handles(self,handles):
        (self.h_camera, self.h_unitlist, self.h_control, self.h_msgdialogue) = handles
    
    def load_level(self, level_id):
        self.level_id = level_id
        self.map_size = resources.initleveldata[level_id].map_size[:]
        self.tilesize = 24
        self.music = resources.initleveldata[level_id].music
        self.tilelayer = resources.initleveldata[level_id].tilelayer[:]
        self.occlayer = resources.initleveldata[level_id].occlayer[:]
        
        self.objectlayer = [[] for i in xrange(self.map_size[0]*self.map_size[1])]
        
        # Special mentor messages in tilemap
        if level_id == "level_tutorial":
            for i in xrange(len(resources.mentor_messages['tiles'])):
                t = resources.mentor_messages['tiles'][i]
                msgs = resources.mentor_messages['msg'][i]
                self.InsertObj(t,i+800)
        
        # start up mentor messages
        if level_id in ["level_tutorial","level_medium001",'level_twosections']:
            self.h_msgdialogue.DisplayMsg('none',mentor=True,mentor_ind=level_id)
        
        # send initial data to camera
        self.h_camera.xlim = self.map_size[0]*self.tilesize
        self.h_camera.ylim = self.map_size[1]*self.tilesize
        
        # Initialise units data and load into object layer
        for i in xrange(len(resources.initleveldata[level_id].units)):
            unit = resources.initleveldata[level_id].units[i]
            self.h_unitlist.append(units.MouseControlableUnit(i,unit[1],unit[2]))
            self.h_unitlist[-1].set_handles((self.h_camera, self, self.parent.msgdialogue, self.h_control))
            self.h_unitlist[-1].SetTile(unit[0])
            self.InsertObj(unit[0],i)
        
        # Initialise doors
        self.doorcontrol = DoorControl(self)
        for door in resources.initleveldata[level_id].doors:
            tile1 = door[0]
            tile2 = door[1]
            state = door[2]
            self.doorcontrol.AddDoor(tile1,tile2,state)
        
        # Initialise Vacuum Sources and vacuum control
        self.vacsources = []
        for v in resources.initleveldata[level_id].vacsources:
            self.vacsources.append(v[:])
        self.airlayer = [1 for i in xrange(self.map_size[0]*self.map_size[1])]
        self.vacuumcontroller = path.VacuumController(self.map_size,self.tilesize,self.occlayer,self.airlayer,self.vacsources)
        self.gfxairlayer = pygame.Surface((self.tilesize*self.map_size[0],self.tilesize*self.map_size[1]))
        self.gfxairlayer.fill((0,0,0))
        self.gfxairlayer.convert()
        self.gfxairlayer.set_alpha(100)
        self.vacuumcontroller.UpdateVacuum([])
        
        # Initialise Vacuum pumps
        self.vacpumpcontrol = VacPumpControl(self)
        for vacpump in resources.initleveldata[level_id].vacpumps:
            tile1 = vacpump[0]
            tile2 = vacpump[1]
            maxvolume = vacpump[2]
            self.vacpumpcontrol.AddVacPump(tile1,tile2,maxvolume)
        
        # Initialise switches
        self.switchcontrol = SwitchControl(self)
        for switch in resources.initleveldata[level_id].switches:
            tile1 = switch[0]
            tile2 = switch[1]
            type = switch[2]
            mode = switch[3]
            powerstate = switch[4]
            h_assoc = []
            for ind in switch[5:]:
                if type == 'door':
                    h_assoc.append(self.doorcontrol.doors[ind])
                elif type == 'pump':
                    h_assoc.append(self.vacpumpcontrol.vacpumps[ind])
                elif type == 'powerplug':
                    h_assoc.append(self.switchcontrol.switches[ind])
            self.switchcontrol.AddSwitch(tile1,tile2,type,mode,powerstate,h_assoc)
        
        # Initialise fires
        self.firecontrol = FireControl(resources.initleveldata[level_id].firetiles,self)
        
        # Initialise power packs
        self.powerpackcontrol = PowerPackControl(resources.initleveldata[level_id].powerpacktiles,self)
        
        # Initialise goal locations
        eb = resources.initleveldata[level_id].goaltiles[0]
        msx = self.map_size[0]
        self.goaltiles = [eb+2-msx,eb+3-msx,eb+4-msx,eb+5-msx,eb+2,eb+3,eb+4,eb+5,eb+2+msx,eb+3+msx,eb+4+msx,eb+5+msx]
        for i in xrange(len(self.goaltiles)):
            self.InsertObj(self.goaltiles[i],i+100)
        
        # escape pod
        self.escapepod = EscapePod(self,resources.initleveldata[level_id].goaltiles[0])
        
        # render tiled layers
        self.bglayer_t = pygame.Surface((self.tilesize*self.map_size[0],self.tilesize*self.map_size[1]))
        self.bglayer_t.fill((255,0,255))
        self.bglayer_t.convert()
        self.bglayer_t.set_colorkey((255,0,255))
        for i in range(self.map_size[0]):
            for j in range(self.map_size[1]):
                coords = [i*self.tilesize, j*self.tilesize]
                tileval = self.tilelayer[self.map_size[0]*j+i]
                if tileval >= 0:
                    tilecoords = resources.tiles_coords[tileval]
                    self.bglayer_t.blit(resources.tiles, (coords[0],coords[1]), area=tilecoords)
        
        self.ReRenderAirLayer()
        
    def UpdateTileLayer(self, tile, tileval):
        coords = [(tile%self.map_size[0])*self.tilesize, (tile/self.map_size[0])*self.tilesize]
        if tileval >= 0:
            tilecoords = resources.tiles_coords[tileval]
            self.bglayer_t.blit(resources.tiles, (coords[0],coords[1]), area=tilecoords)
    
    def RenderTileLayer(self, screen):
        screen.blit(self.bglayer_t, (0,0), area=(self.h_camera.x-self.h_camera.w_view/2, self.h_camera.y-self.h_camera.h_view/2, self.h_camera.w_view, self.h_camera.h_view))
    
    def ReRenderAirLayer(self):
        msx = self.map_size[0]
        self.gfxairlayer.fill((255,0,0))
        for i in range(self.map_size[0]):
            for j in range(self.map_size[1]):
                tile = self.map_size[0]*j+i
                if self.occlayer[tile] == 0 and self.tilelayer[tile] >= 0 and self.airlayer[tile] == 1:
                    coords = [i*self.tilesize, j*self.tilesize]
                    pygame.draw.rect(self.gfxairlayer,(0,255,0),((coords[0],coords[1]),(self.tilesize,self.tilesize)))
        for i in range(self.map_size[0]):
            for j in range(self.map_size[1]):
                coords = [i*self.tilesize, j*self.tilesize]
                tile = self.map_size[0]*j+i
                tileval = self.tilelayer[tile]
                if tileval >= 0:
                    #print len(resources.tilevaclookup), tileval
                    darkind = resources.tilevaclookup[tileval]
                    tilevaccheck = [-1-msx,-msx,1-msx,-1,0,1,-1+msx,msx,1+msx][darkind-1]
                    if self.airlayer[tile+tilevaccheck] == 0:
                        tileval += 4 # use dark tile
                    tilecoords = resources.tiles_coords[tileval]
                    self.bglayer_t.blit(resources.tiles, (coords[0],coords[1]), area=tilecoords)
        for door in self.doorcontrol.doors:
            for t in door.tilelist:
                if self.airlayer[t+msx] == 0 and self.airlayer[t-msx] == 0: # trapped air inside closed door, covered by vacuum
                    tileval = self.tilelayer[t-1]+4
                    tilecoords = resources.tiles_coords[tileval]
                    coords = [((t-1)%msx)*self.tilesize, ((t-1)/msx)*self.tilesize]
                    self.bglayer_t.blit(resources.tiles, (coords[0],coords[1]), area=tilecoords)
                    tileval = self.tilelayer[t+1]+4
                    tilecoords = resources.tiles_coords[tileval]
                    coords = [((t+1)%msx)*self.tilesize, ((t+1)/msx)*self.tilesize]
                    self.bglayer_t.blit(resources.tiles, (coords[0],coords[1]), area=tilecoords)
    
    def RenderAirLayer(self, screen):
        screen.blit(self.gfxairlayer, (0,0), area=(self.h_camera.x-self.h_camera.w_view/2, self.h_camera.y-self.h_camera.h_view/2, self.h_camera.w_view, self.h_camera.h_view))
        
    def InsertObj(self,tileind,objid):
        self.objectlayer[tileind].append(objid)
    
    def RemoveObj(self,tileind,objid):
        if objid in self.objectlayer[tileind]:
            self.objectlayer[tileind].pop(self.objectlayer[tileind].index(objid))
    
    def UpdateObj(self,tileind_old,tileind_new,objid):
        self.RemoveObj(tileind_old,objid)
        self.InsertObj(tileind_new,objid)
    
    def FindObjects(self,tile,type):
        # Object layer reference:
        #
        # 0-100: units
        # 100-199: ???
        if type == 'units':
            minid = 0
            maxid = 99
        elif type == 'goal':
            minid = 100
            maxid = 199
        elif type == 'fire':
            minid = 200
            maxid = 299
        elif type == 'vacsource':
            minid = 300
            maxid = 399
        elif type == 'switch':
            minid = 400
            maxid = 499
        elif type == 'door':
            minid = 500
            maxid = 599
        elif type == 'pump':
            minid = 600
            maxid = 699
        elif type == 'key':
            minid = 700
            maxid = 799
        elif type == 'mentor':
            minid = 800
            maxid = 899
        idlist = []
        for id in self.objectlayer[tile]:
            if id >= minid and id < maxid:
                idlist.append(id-minid)
        return idlist
    
    def Update(self):
        self.firecontrol.UpdateFires()
        for p in self.vacpumpcontrol.vacpumps:
            p.Update()
        self.escapepod.Update()
        #self.doorcontrol.UpdateDoors()
    